@@ -1,19 +1,18 @@
 /*
  * See Licensing and Copyright notice in naev.h
  */
-
 /*
  * compliant with rfc3548
  */
+/** @cond */
+#include <stdint.h>
+#include <stdlib.h>
+/** @endcond */
 
 #include "base64.h"
 
-#include <stdlib.h>
-#include <stdint.h>
-
 #include "nstring.h"
 
-
 /* encode table - base 64 alphabet as defined by the rfc */
 static const char cb64[64]="ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/";
 /* generate decode table at compile time */
@@ -150,7 +149,6 @@ static const signed char cd64[256] = { /* makes it much faster */
       B64 (252), B64 (253), B64 (254), B64 (255)
 };
 
-
 /*
  * encodes src of sz length storing the new length in len
  */
@@ -203,7 +201,6 @@ char* base64_encode( size_t *len, const char *src, size_t sz )
    return r;
 }
 
-
 /*
  * decode the buffer, same syntax as base64_encode
  */
@@ -213,7 +210,6 @@ char* base64_decode( size_t *len, const char *src, size_t sz )
 {
    char *r, *dat, pad;
    size_t c,i,j;
-   uint32_t n;
 
    /* allocate r */
    c = sz * 3 / 4 + 2;
@@ -233,9 +229,8 @@ char* base64_decode( size_t *len, const char *src, size_t sz )
    /* fill r */
    i = 0;
    for (c=0; c<j; c+=4) {
-
       /* process the input from base 64 */
-      n = dec_ch( dat[c+0] ) << 18; /* guaranteed to be valid */
+      uint32_t n = dec_ch( dat[c+0] ) << 18; /* guaranteed to be valid */
       n += (c+1<j) ? (dec_ch( dat[c+1] ) << 12) : 0; /* check if inbounds */
       n += (c+2<j) ? (dec_ch( dat[c+2] ) << 6)  : 0;
       n += (c+3<j) ? (dec_ch( dat[c+3] ) << 0)  : 0;
@@ -253,7 +248,6 @@ char* base64_decode( size_t *len, const char *src, size_t sz )
    return r;
 }
 
-
 /*
  * encodes src of sz length storing the new length in len
  */
@@ -263,7 +257,6 @@ char* base64_encode_to_cstr( const char *src, size_t sz )
    return base64_encode( &len, src, sz );
 }
 
-
 /*
  * encodes src of sz length storing the new length in len
  */
