@@ -1,29 +1,30 @@
 /*
  * See Licensing and Copyright notice in naev.h
  */
-
 /**
  * @file collision.c
  *
  * @brief Deals with 2d collisions.
  */
 
+/** @cond */
+#include "naev.h"
+
+#include "SDL.h"
+/** @endcond */
 
 #include "collision.h"
 
-#include "naev.h"
-
 #include "log.h"
-
+#include "array.h"
 
 /*
  * Prototypes
  */
-static int pointInPolygon( const CollPoly* at, const Vector2d* ap,
+static int PointInPolygon( const CollPoly* at, const vec2* ap,
       float x, float y );
-static int LineOnPolygon( const CollPoly* at, const Vector2d* ap,
-      float x1, float y1, float x2, float y2, Vector2d* crash );
-
+static int LineOnPolygon( const CollPoly* at, const vec2* ap,
+      float x1, float y1, float x2, float y2, vec2* crash );
 
 /**
  * @brief Loads a polygon from an xml node.
@@ -33,55 +34,54 @@ static int LineOnPolygon( const CollPoly* at, const Vector2d* ap,
  */
 void LoadPolygon( CollPoly* polygon, xmlNodePtr node )
 {
-   float d;
-   xmlNodePtr cur;
-   char *list, *ch;
-   int i;
-
-   cur = node->children;
+   xmlNodePtr cur = node->children;
    do {
       if (xml_isNode(cur,"x")) {
-         list = xml_get(cur);
-         i = 0;
+         char *saveptr;
+         char *list = xml_get(cur);
          /* split the list of coordiantes */
-         ch = strtok(list, ",");
-         polygon->x = malloc( sizeof(float) );
+         char *ch = SDL_strtokr(list, ",", &saveptr);
+         polygon->x = array_create_size(float, 32);
          polygon->xmin = 0;
          polygon->xmax = 0;
-         while ( ch != NULL ) {
-            i++;
-            polygon->x = realloc( polygon->x, sizeof(float) * i );
-            d = atof(ch);
-            polygon->x[i-1] = d;
+         while (ch != NULL) {
+            float d = atof(ch);
+            array_push_back( &polygon->x, d );
             polygon->xmin = MIN( polygon->xmin, d );
             polygon->xmax = MAX( polygon->xmax, d );
-            ch = strtok(NULL, ",");
+            ch = SDL_strtokr(NULL, ",", &saveptr);
          }
       }
       else if (xml_isNode(cur,"y")) {
-         list = xml_get(cur);
-         i = 0;
+         char *saveptr;
+         char *list = xml_get(cur);
          /* split the list of coordiantes */
-         ch = strtok(list, ",");
-         polygon->y = malloc( sizeof(float) );
+         char *ch = SDL_strtokr(list, ",", &saveptr);
+         polygon->y = array_create_size(float, 32);
          polygon->ymin = 0;
          polygon->ymax = 0;
-         while ( ch != NULL ) {
-            i++;
-            polygon->y = realloc( polygon->y, sizeof(float) * i );
-            d = atof(ch);
-            polygon->y[i-1] = d;
+         while (ch != NULL) {
+            float d = atof(ch);
+            array_push_back( &polygon->y, d );
             polygon->ymin = MIN( polygon->ymin, d );
             polygon->ymax = MAX( polygon->ymax, d );
-            ch = strtok(NULL, ",");
+            ch = SDL_strtokr(NULL, ",", &saveptr);
          }
-         polygon->npt = i;
       }
    } while (xml_nextNode(cur));
 
+   polygon->npt = array_size(polygon->x);
+   if (array_size(polygon->y) != polygon->npt)
+      WARN(_("Polygon with mismatch of number of |x|=%d and |y|=%d coordinates detected!"), polygon->npt, array_size(polygon->y) );
+
    return;
 }
 
+void FreePolygon( CollPoly *polygon )
+{
+   array_free( polygon->x );
+   array_free( polygon->y );
+}
 
 /**
  * @brief Checks whether or not two sprites collide.
@@ -100,9 +100,9 @@ void LoadPolygon( CollPoly* polygon, xmlNodePtr node )
  *    @param[out] crash Actual position of the collision (only set on collision).
  *    @return 1 on collision, 0 else.
  */
-int CollideSprite( const glTexture* at, const int asx, const int asy, const Vector2d* ap,
-      const glTexture* bt, const int bsx, const int bsy, const Vector2d* bp,
-      Vector2d* crash )
+int CollideSprite( const glTexture* at, const int asx, const int asy, const vec2* ap,
+      const glTexture* bt, const int bsx, const int bsy, const vec2* bp,
+      vec2* crash )
 {
    int x,y;
    int ax1,ax2, ay1,ay2;
@@ -170,7 +170,6 @@ int CollideSprite( const glTexture* at, const int asx, const int asy, const Vect
    return 0;
 }
 
-
 /**
  * @brief Checks whether or not a sprite collides with a polygon.
  *
@@ -183,9 +182,9 @@ int CollideSprite( const glTexture* at, const int asx, const int asy, const Vect
  *    @param[out] crash Actual position of the collision (only set on collision).
  *    @return 1 on collision, 0 else.
  */
-int CollideSpritePolygon( const CollPoly* at, const Vector2d* ap,
-      const glTexture* bt, const int bsx, const int bsy, const Vector2d* bp,
-      Vector2d* crash )
+int CollideSpritePolygon( const CollPoly* at, const vec2* ap,
+      const glTexture* bt, const int bsx, const int bsy, const vec2* bp,
+      vec2* crash )
 {
    int x,y;
    int ax1,ax2, ay1,ay2;
@@ -234,7 +233,7 @@ int CollideSpritePolygon( const CollPoly* at, const Vector2d* ap,
       for (x=inter_x0; x<=inter_x1; x++) {
          /* compute offsets for surface before pass to TransparentPixel test */
          if ((!gl_isTrans(bt, bbx + x, bby + y))) {
-            if (pointInPolygon( at, ap, (float)x, (float)y )) {
+            if (PointInPolygon( at, ap, (float)x, (float)y )) {
                crash->x = x;
                crash->y = y;
                return 1;
@@ -246,7 +245,6 @@ int CollideSpritePolygon( const CollPoly* at, const Vector2d* ap,
    return 0;
 }
 
-
 /**
  * @brief Checks whether or not two polygons collide.
  *  /!\ The function is not symmetric: the points of polygon 2 are tested
@@ -259,14 +257,13 @@ int CollideSpritePolygon( const CollPoly* at, const Vector2d* ap,
  *    @param[out] crash Actual position of the collision (only set on collision).
  *    @return 1 on collision, 0 else.
  */
-int CollidePolygon( const CollPoly* at, const Vector2d* ap,
-      const CollPoly* bt, const Vector2d* bp, Vector2d* crash )
+int CollidePolygon( const CollPoly* at, const vec2* ap,
+      const CollPoly* bt, const vec2* bp, vec2* crash )
 {
-   int i;
    int ax1,ax2, ay1,ay2;
    int bx1,bx2, by1,by2;
    int inter_x0, inter_x1, inter_y0, inter_y1;
-   float xabs, yabs, x1, y1, x2, y2;
+   float x1, y1, x2, y2;
 
    /* a - cube coordinates */
    ax1 = (int)VX(*ap) + (int)(at->xmin);
@@ -291,13 +288,14 @@ int CollidePolygon( const CollPoly* at, const Vector2d* ap,
    inter_y1 = MIN( ay2, by2 );
 
    /* loop on the points of bt to see if one of them is in polygon at. */
-   for (i=0; i<=bt->npt-1; i++) {
+   for (int i=0; i<=bt->npt-1; i++) {
+      float xabs, yabs;
       xabs = bt->x[i] + VX(*bp);
       yabs = bt->y[i] + VY(*bp);
 
       if ((xabs<inter_x0) || (xabs>inter_x1) ||
           (yabs<inter_y0) || (yabs>inter_y1)) {
-         if (pointInPolygon( at, ap, xabs, yabs )) {
+         if (PointInPolygon( at, ap, xabs, yabs )) {
             crash->x = (int)xabs;
             crash->y = (int)yabs;
             return 1;
@@ -310,9 +308,9 @@ int CollidePolygon( const CollPoly* at, const Vector2d* ap,
    y1 = bt->y[0] + VY(*bp);
    x2 = bt->x[bt->npt-1] + VX(*bp);
    y2 = bt->y[bt->npt-1] + VY(*bp);
-   if ( LineOnPolygon( at, ap, x1, y1, x2, y2, crash ) )
+   if (LineOnPolygon( at, ap, x1, y1, x2, y2, crash ))
       return 1;
-   for (i=0; i<=bt->npt-2; i++) {
+   for (int i=0; i<=bt->npt-2; i++) {
       x1 = bt->x[i] + VX(*bp);
       y1 = bt->y[i] + VY(*bp);
       x2 = bt->x[i+1] + VX(*bp);
@@ -324,6 +322,40 @@ int CollidePolygon( const CollPoly* at, const Vector2d* ap,
    return 0;
 }
 
+/**
+ * @brief Rotates a polygon.
+ *
+ *    @param[out] rpolygon Rotated polygon.
+ *    @param[in] ipolygon Imput polygon.
+ *    @param[in] theta Rotation angle (radian).
+ */
+void RotatePolygon( CollPoly* rpolygon, CollPoly* ipolygon, float theta )
+{
+   float ct, st;
+
+   rpolygon->npt = ipolygon->npt;
+   rpolygon->x = malloc( ipolygon->npt*sizeof(float) );
+   rpolygon->y = malloc( ipolygon->npt*sizeof(float) );
+   rpolygon->xmin = 0;
+   rpolygon->xmax = 0;
+   rpolygon->ymin = 0;
+   rpolygon->ymax = 0;
+
+   ct = cos( theta );
+   st = sin( theta );
+
+   for (int i=0; i<=rpolygon->npt-1; i++) {
+      float d = ipolygon->x[i] * ct - ipolygon->y[i] * st;
+      rpolygon->x[i] = d;
+      rpolygon->xmin = MIN( rpolygon->xmin, d );
+      rpolygon->xmax = MAX( rpolygon->xmax, d );
+
+      d = ipolygon->x[i] * st + ipolygon->y[i] * ct ;
+      rpolygon->y[i] = d;
+      rpolygon->ymin = MIN( rpolygon->ymin, d );
+      rpolygon->ymax = MAX( rpolygon->ymax, d );
+   }
+}
 
 /**
  * @brief Checks whether or not a point is inside a polygon.
@@ -334,10 +366,9 @@ int CollidePolygon( const CollPoly* at, const Vector2d* ap,
  *    @param[in] y Coordinate of point.
  *    @return 1 on collision, 0 else.
  */
-int pointInPolygon( const CollPoly* at, const Vector2d* ap,
+static int PointInPolygon( const CollPoly* at, const vec2* ap,
       float x, float y )
 {
-   int i;
    float vprod, sprod, angle;
    float dxi, dxip, dyi, dyip;
 
@@ -346,7 +377,7 @@ int pointInPolygon( const CollPoly* at, const Vector2d* ap,
       If the final angle is 0, we are outside the polygon
       Otherwise, the angle should be +/- 2pi*/
    angle = 0.0;
-   for (i=0; i<=at->npt-2; i++) {
+   for (int i=0; i<=at->npt-2; i++) {
       dxi  = at->x[i]  +VX(*ap)-x;
       dxip = at->x[i+1]+VX(*ap)-x;
       dyi  = at->y[i]  +VY(*ap)-y;
@@ -369,7 +400,6 @@ int pointInPolygon( const CollPoly* at, const Vector2d* ap,
    return 1;
 }
 
-
 /**
  * @brief Checks whether or not a line intersects a polygon.
  *
@@ -382,11 +412,10 @@ int pointInPolygon( const CollPoly* at, const Vector2d* ap,
  *    @param[out] crash coordinates of the intersection.
  *    @return 1 on collision, 0 else.
  */
-int LineOnPolygon( const CollPoly* at, const Vector2d* ap,
-      float x1, float y1, float x2, float y2, Vector2d* crash )
+static int LineOnPolygon( const CollPoly* at, const vec2* ap,
+      float x1, float y1, float x2, float y2, vec2* crash )
 {
    float xi, xip, yi, yip;
-   int i;
 
    /* In this function, we are only looking for one collision point. */
 
@@ -396,7 +425,7 @@ int LineOnPolygon( const CollPoly* at, const Vector2d* ap,
    yip = at->y[0]         + ap->y;
    if ( CollideLineLine(x1, y1, x2, y2, xi, yi, xip, yip, crash) == 1 )
       return 1;
-   for (i=0; i<=at->npt-2; i++) {
+   for (int i=0; i<=at->npt-2; i++) {
       xi  = at->x[i]   + ap->x;
       xip = at->x[i+1] + ap->x;
       yi  = at->y[i]   + ap->y;
@@ -408,7 +437,6 @@ int LineOnPolygon( const CollPoly* at, const Vector2d* ap,
    return 0;
 }
 
-
 /**
  * @brief Checks to see if two lines collide.
  *
@@ -425,16 +453,16 @@ int LineOnPolygon( const CollPoly* at, const Vector2d* ap,
  *              1 if lines just collide on a point, or 0 if they don't.
  */
 int CollideLineLine( double s1x, double s1y, double e1x, double e1y,
-      double s2x, double s2y, double e2x, double e2y, Vector2d* crash )
+      double s2x, double s2y, double e2x, double e2y, vec2* crash )
 {
    double ua_t, ub_t, u_b;
-   double ua, ub;
 
    ua_t = (e2x - s2x) * (s1y - s2y) - (e2y - s2y) * (s1x - s2x);
    ub_t = (e1x - s1x) * (s1y - s2y) - (e1y - s1y) * (s1x - s2x);
    u_b  = (e2y - s2y) * (e1x - s1x) - (e2x - s2x) * (e1y - s1y);
 
    if (u_b != 0.) {
+      double ua, ub;
       ua = ua_t / u_b;
       ub = ub_t / u_b;
 
@@ -458,7 +486,6 @@ int CollideLineLine( double s1x, double s1y, double e1x, double e1y,
    }
 }
 
-
 /**
  * @brief Checks to see if a line collides with a sprite.
  *
@@ -478,14 +505,14 @@ int CollideLineLine( double s1x, double s1y, double e1x, double e1y,
  *
  * @sa CollideSprite
  */
-int CollideLineSprite( const Vector2d* ap, double ad, double al,
-      const glTexture* bt, const int bsx, const int bsy, const Vector2d* bp,
-      Vector2d crash[2] )
+int CollideLineSprite( const vec2* ap, double ad, double al,
+      const glTexture* bt, const int bsx, const int bsy, const vec2* bp,
+      vec2 crash[2] )
 {
    int x,y, rbsy, bbx,bby;
    double ep[2], bl[2], tr[2], v[2], mod;
    int hits, real_hits;
-   Vector2d tmp_crash, border[2];
+   vec2 tmp_crash, border[2];
 
    /* Make sure texture has transparency map. */
    if (bt->trans == NULL) {
@@ -611,7 +638,6 @@ int CollideLineSprite( const Vector2d* ap, double ad, double al,
    return 1;
 }
 
-
 /**
  * @brief Checks to see if a line collides with a polygon.
  *
@@ -628,14 +654,13 @@ int CollideLineSprite( const Vector2d* ap, double ad, double al,
  *
  * @sa CollideLinePolygon
  */
-int CollideLinePolygon( const Vector2d* ap, double ad, double al,
-      const CollPoly* bt, const Vector2d* bp, Vector2d crash[2] )
+int CollideLinePolygon( const vec2* ap, double ad, double al,
+      const CollPoly* bt, const vec2* bp, vec2 crash[2] )
 {
-   int i;
-   double ep[2], bl[2], tr[2];
+   double ep[2];
    double xi, yi, xip, yip;
-   int hits, real_hits;
-   Vector2d tmp_crash;
+   int real_hits;
+   vec2 tmp_crash;
 
    /* Set up end point of line. */
    ep[0] = ap->x + al*cos(ad);
@@ -645,14 +670,14 @@ int CollideLinePolygon( const Vector2d* ap, double ad, double al,
    vectnull( &tmp_crash );
 
    /* Check if the beginning point is inside polygon */
-   if ( pointInPolygon( bt, bp, (float) ap->x, (float) ap->y ) ) {
+   if (PointInPolygon( bt, bp, (float) ap->x, (float) ap->y )) {
       crash[real_hits].x = ap->x;
       crash[real_hits].y = ap->y;
       real_hits++;
    }
 
    /* same thing for end point */
-   if ( pointInPolygon( bt, bp, (float) ep[0], (float) ep[1] ) ) {
+   if (PointInPolygon( bt, bp, (float) ep[0], (float) ep[1] )) {
       crash[real_hits].x = ep[0];
       crash[real_hits].y = ep[1];
       real_hits++;
@@ -664,6 +689,8 @@ int CollideLinePolygon( const Vector2d* ap, double ad, double al,
 
    /* None is inside, check if there is a chance of intersection */
    if (real_hits == 0) {
+      int hits;
+      double bl[2], tr[2];
       /* Set up top right corner of the rectangle. */
       tr[0] = bp->x + (double)bt->xmax;
       tr[1] = bp->y + (double)bt->ymax;
@@ -681,17 +708,17 @@ int CollideLinePolygon( const Vector2d* ap, double ad, double al,
          hits++;
 
       /* Top border. */
-      if (CollideLineLine(ap->x, ap->y, ep[0], ep[1],
+      if (!hits && CollideLineLine(ap->x, ap->y, ep[0], ep[1],
             bl[0], tr[1], tr[0], tr[1], &tmp_crash) == 1)
          hits++;
 
       /* Right border. */
-      if (CollideLineLine(ap->x, ap->y, ep[0], ep[1],
+      if (!hits && CollideLineLine(ap->x, ap->y, ep[0], ep[1],
             tr[0], tr[1], tr[0], bl[1], &tmp_crash) == 1)
          hits++;
 
       /* Bottom border. */
-      if ((hits < 2) && CollideLineLine(ap->x, ap->y, ep[0], ep[1],
+      if (!hits && CollideLineLine(ap->x, ap->y, ep[0], ep[1],
             tr[0], bl[1], bl[0], bl[1], &tmp_crash) == 1)
          hits++;
 
@@ -715,7 +742,7 @@ int CollideLinePolygon( const Vector2d* ap, double ad, double al,
       if (real_hits == 2)
          return 1;
    }
-   for (i=0; i<=bt->npt-2; i++) {
+   for (int i=0; i<=bt->npt-2; i++) {
       xi  = (double)bt->x[i]   + bp->x;
       xip = (double)bt->x[i+1] + bp->x;
       yi  = (double)bt->y[i]   + bp->y;
@@ -744,3 +771,333 @@ int CollideLinePolygon( const Vector2d* ap, double ad, double al,
    /* We hit. */
    return 1;
 }
+
+/**
+ * @brief Checks to see if a circle collides with a polygon.
+ *
+ * First collisions are detected on all the walls of the polygon's rectangle.
+ *  Then the collisions are tested on every line of the polygon.
+ *
+ *    @param[in] ap Origin of the circle.
+ *    @param[in] ar Radius of the circle.
+ *    @param[in] bt Polygon b.
+ *    @param[in] bp Position in space of polygon b.
+ *    @param[out] crash Position of the collision.
+ *    @return 1 on collision, 0 else.
+ *
+ * @sa CollideCirclePolygon
+ */
+int CollideCirclePolygon( const vec2* ap, double ar,
+      const CollPoly* bt, const vec2* bp, vec2 crash[2] )
+{
+	vec2 p1, p2;
+   int real_hits;
+   vec2 tmp_crash[2];
+
+   real_hits = 0;
+   vectnull( &tmp_crash[0] );
+   vectnull( &tmp_crash[1] );
+
+   /* Set up top right corner of the rectangle. */
+   p1.x = bp->x + (double)bt->xmax;
+   p1.y = bp->y + (double)bt->ymax;
+   /* Set up bottom left corner of the rectangle. */
+   p2.x = bp->x + (double)bt->xmin;
+   p2.y = bp->y + (double)bt->ymin;
+
+   /* Start check for rectangular collisions. */
+   if ((ap->x-ar > p1.x) && (ap->x+ar < p2.x) &&
+         (ap->y-ar > p1.y) && (ap->y+ar < p2.y))
+      return 0;
+
+   /*
+    * Now we check any line of the polygon
+    */
+   p1.x = (double)bt->x[bt->npt-1] + bp->x;
+   p2.x = (double)bt->x[0]         + bp->x;
+   p1.y = (double)bt->y[bt->npt-1] + bp->y;
+   p2.y = (double)bt->y[0]         + bp->y;
+   if (CollideLineCircle( &p1, &p2, ap, ar, tmp_crash )) {
+      crash[real_hits].x = tmp_crash[0].x;
+      crash[real_hits].y = tmp_crash[0].y;
+      real_hits++;
+      if (real_hits == 2)
+         return 1;
+   }
+   for (int i=0; i<=bt->npt-2; i++) {
+      p1.x = (double)bt->x[i]   + bp->x;
+      p2.x = (double)bt->x[i+1] + bp->x;
+      p1.y = (double)bt->y[i]   + bp->y;
+      p2.y = (double)bt->y[i+1] + bp->y;
+		if (CollideLineCircle( &p1, &p2, ap, ar, tmp_crash )) {
+         crash[real_hits].x = tmp_crash[0].x;
+         crash[real_hits].y = tmp_crash[0].y;
+         real_hits++;
+         if (real_hits == 2)
+            return 1;
+      }
+   }
+
+   /* Actually missed. */
+   if (real_hits == 0)
+      return 0;
+
+   /* Strange situation, should never happen but just in case we duplicate
+    *  the hit. */
+   if (real_hits == 1) {
+      crash[1].x = crash[0].x;
+      crash[1].y = crash[0].y;
+   }
+
+   /* We hit. */
+   return 1;
+}
+
+/**
+ * @brief Checks whether or not a sprite collides with a polygon.
+ *
+ *    @param[in] ap Position in space of circle a.
+ *    @param[in] ar Radius of circle a.
+ *    @param[in] bt Texture b.
+ *    @param[in] bsx Position of x of sprite b.
+ *    @param[in] bsy Position of y of sprite b.
+ *    @param[in] bp Position in space of sprite b.
+ *    @param[out] crash Actual position of the collision (only set on collision).
+ *    @return 1 on collision, 0 else.
+ */
+int CollideCircleSprite( const vec2 *ap, double ar, const glTexture* bt,
+      const int bsx, const int bsy, const vec2* bp,vec2* crash )
+{
+   int r, acx,acy, ax1,ax2, ay1,ay2;
+   int bx1,bx2, by1,by2;
+   int inter_x0, inter_x1, inter_y0, inter_y1;
+   int rbsy;
+   int bbx, bby;
+
+#if DEBUGGING
+   /* Make sure the surfaces have transparency maps. */
+   if (bt->trans == NULL) {
+      WARN(_("Texture '%s' has no transparency map"), bt->name);
+      return 0;
+   }
+#endif /* DEBUGGING */
+
+   /* a - cube coordinates */
+   r = ceil(ar);
+   acx = (int) VX(*ap);
+   acy = (int) VY(*ap);
+   ax1 = acx - r;
+   ay1 = acy - r;
+   ax2 = acx + r;
+   ay2 = acy + r;
+
+   /* b - cube coordinates */
+   bx1 = (int)VX(*bp) - (int)(bt->sw)/2;
+   by1 = (int)VY(*bp) - (int)(bt->sh)/2;
+   bx2 = bx1 + bt->sw - 1;
+   by2 = by1 + bt->sh - 1;
+
+   /* check if bounding boxes intersect */
+   if ((bx2 < ax1) || (ax2 < bx1)) return 0;
+   if ((by2 < ay1) || (ay2 < by1)) return 0;
+
+   /* define the remaining binding box */
+   inter_x0 = MAX( ax1, bx1 );
+   inter_x1 = MIN( ax2, bx2 );
+   inter_y0 = MAX( ay1, by1 );
+   inter_y1 = MIN( ay2, by2 );
+
+   /* real vertical sprite value (flipped) */
+   rbsy = bt->sy - bsy - 1;
+
+   /* set up the base points */
+   bbx =  bsx*(int)(bt->sw) - bx1;
+   bby = rbsy*(int)(bt->sh) - by1;
+   for (int y=inter_y0; y<=inter_y1; y++) {
+      for (int x=inter_x0; x<=inter_x1; x++) {
+         /* compute offsets for surface before pass to TransparentPixel test */
+         if ((!gl_isTrans(bt, bbx + x, bby + y))) {
+            if (pow2(x-acx)+pow2(y-acy) <= r*r) {
+               crash->x = x;
+               crash->y = y;
+               return 1;
+            }
+         }
+      }
+   }
+
+   return 0;
+}
+
+static int linePointOnSegment( double d1, double x1, double y1, double x2, double y2, double x, double y )
+{
+   //double d1 = hypot( x2-x1, y2-y1 ); /* Distance between end-points. */
+   double d2 = hypot( x-x1,  y-y1 );  /* Distance from point to one end. */
+   double d3 = hypot( x2-x,  y2-y );  /* Distance to the other end. */
+   return fabs(d1 - d2 - d3) < 1e-8;   /* True if smaller than some tolerance. */
+}
+
+#define FX( A, B, C, x )   (-(A * x + C) / B)
+#define FY( A, B, C , y )  (-(B * y + C) / A)
+/**
+ * @brief Checks to see if a line collides with a circle
+ *
+ *    @param[in] p1 Point 1 of the line segment.
+ *    @param[in] p2 Point 2 of the line segment.
+ *    @param[in] cc Center of the circle.
+ *    @param[in] cr Radius of the circle.
+ *    @param[out] crash Position of the collision.
+ *    @return 1 on collision, 0 else.
+ */
+int CollideLineCircle( const vec2* p1, const vec2* p2,
+      const vec2 *cc, double cr, vec2 crash[2] )
+{
+   double x0 = cc->x;
+   double y0 = cc->y;
+   double x1 = p1->x;
+   double y1 = p1->y;
+   double x2 = p2->x;
+   double y2 = p2->y;
+
+   double A = y2 - y1;
+   double B = x1 - x2;
+   double C = x2 * y1 - x1 * y2;
+
+   double a = pow2(A) + pow2(B);
+   double b, c, d;
+
+   int bnz, cnt;
+
+   double x, y, d1;
+
+   /* Non-vertical case. */
+   if (fabs(B) >= 1e-8) {
+      b = 2. * (A * C + A * B * y0 - pow2(B) * x0);
+      c = pow2(C) + 2. * B * C * y0 - pow2(B) * (pow2(cr) - pow2(x0) - pow2(y0));
+      bnz = 1;
+   }
+   /* Have to have special care when line is vertical. */
+   else {
+      b = 2. * (B * C + A * B * x0 - pow2(A) * y0);
+      c = pow2(C) + 2. * A * C * x0 - pow2(A) * (pow2(cr) - pow2(x0) - pow2(y0));
+      bnz = 0;
+   }
+   d = pow2(b) - 4. * a * c; /* Discriminant. */
+   if (d < 0.)
+      return 0;
+
+   cnt = 0;
+   d1 = hypot( x2-x1, y2-y1 );
+   /* Line is tangent, so only one intersection. */
+   if (d == 0.) {
+      if (bnz) {
+         x = -b / (2. * a);
+         y = FX(A, B, C, x);
+         if (linePointOnSegment( d1, x1, y1, x2, y2, x, y )) {
+            crash[cnt].x = x;
+            crash[cnt].y = y;
+            cnt++;
+         }
+      } else {
+         y = -b / (2. * a);
+         x = FY(A, B, C, y);
+         if (linePointOnSegment( d1, x1, y1, x2, y2, x, y )) {
+            crash[cnt].x = x;
+            crash[cnt].y = y;
+            cnt++;
+         }
+      }
+   }
+   /* Two intersections. */
+   else {
+      d = sqrt(d);
+      if (bnz) {
+         x = (-b + d) / (2. * a);
+         y = FX(A, B, C, x);
+         if (linePointOnSegment( d1, x1, y1, x2, y2, x, y )) {
+            crash[cnt].x = x;
+            crash[cnt].y = y;
+            cnt++;
+         }
+         x = (-b - d) / (2. * a);
+         y = FX(A, B, C, x);
+         if (linePointOnSegment( d1, x1, y1, x2, y2, x, y )) {
+            crash[cnt].x = x;
+            crash[cnt].y = y;
+            cnt++;
+         }
+      } else {
+         y = (-b + d) / (2. * a);
+         x = FY(A, B, C, y);
+         if (linePointOnSegment( d1, x1, y1, x2, y2, x, y )) {
+            crash[cnt].x = x;
+            crash[cnt].y = y;
+            cnt++;
+         }
+         y = (-b - d) / (2. * a);
+         x = FY(A, B, C, y);
+         if (linePointOnSegment( d1, x1, y1, x2, y2, x, y )) {
+            crash[cnt].x = x;
+            crash[cnt].y = y;
+            cnt++;
+         }
+      }
+   }
+   return cnt;
+}
+#undef FX
+#undef FY
+
+/**
+ * @brief Computes the collision between two circles.
+ */
+int CollideCircleCircle( const vec2 *p1, double r1,
+      const vec2 *p2, double r2, vec2 crash[2] )
+{
+   double dist2 = vec2_dist2( p1, p2 );
+
+   /* No intersection. */
+   if (dist2 > pow2(r1+r2))
+      return 0;
+
+   crash->x = (p1->x * r1 + p2->x * r2) / (r1+r2);
+   crash->y = (p1->y * r1 + p2->y * r2) / (r1+r2);
+   return 1;
+}
+
+/**
+ * @brief Calculates the area of intersection between two circles.
+ */
+double CollideCircleIntersection( const vec2 *p1, double r1,
+      const vec2 *p2, double r2 )
+{
+   double dist2 = vec2_dist2( p1, p2 );
+
+   /* No intersection. */
+   if (dist2 > pow2(r1+r2))
+      return 0.;
+
+   /* One inside the other. */
+   if (dist2 <= pow2(fabs(r1-r2)))
+      return M_PI * pow2( MIN(r1, r2) );
+
+#if 0
+   /* Case exactly the same. */
+   if ((dist2==0.) && (r1==r2))
+      return M_PI * pow2(r1);
+#endif
+
+   /* Distances. */
+   double dist = sqrt( dist2 );
+   double distc1 = (pow2(r1) - pow2(r2) + dist2) / (2. * dist); /* First center point to middle line. */
+   double distc2 = dist - distc1; /* Second center point to middle line. */
+   double height = sqrt( pow2(r1) - pow2(distc1) ); /* Half of middle line. */
+   /* Angles. */
+   double ang1 = fmod( atan2( height, distc1 ) * 2. + 2.*M_PI, 2.*M_PI ); /* Center angle for first circle. */
+   double ang2 = fmod( atan2( height, distc2 ) * 2. + 2.*M_PI, 2.*M_PI ); /*< Center angle for second circle. */
+   /* Areas. */
+   double A1 = pow2(r1) / 2.0 * (ang1 - sin(ang1)); /* Area of first circula segment. */
+   double A2 = pow2(r2) / 2.0 * (ang2 - sin(ang2)); /* Area of second circula segment. */
+
+   return A1+A2;
+}
