@@ -1,7 +1,6 @@
 /*
  * See Licensing and Copyright notice in naev.h
  */
-
 /**
  * @file event.c
  *
@@ -11,47 +10,44 @@
  *  or how they happen.  They can simple do something simple or actually lead up
  *  to and open an entire set of events.
  */
+/** @cond */
+#include "nstring.h"
+#include <stdint.h>
+#include <stdlib.h>
 
+#include "naev.h"
+/** @endcond */
 
 #include "event.h"
 
-#include "naev.h"
-
-#include <stdint.h>
-#include "nstring.h"
-#include <stdlib.h>
-
-#include "log.h"
+#include "conf.h"
 #include "array.h"
-#include "nlua.h"
-#include "nluadef.h"
-#include "nlua_evt.h"
-#include "nlua_hook.h"
-#include "nlua_tk.h"
-#include "nlua_camera.h"
-#include "nlua_bkg.h"
-#include "nlua_tex.h"
-#include "nlua_music.h"
-#include "nlua_spfx.h"
-#include "rng.h"
-#include "ndata.h"
-#include "nxml.h"
-#include "nxml_lua.h"
 #include "cond.h"
 #include "hook.h"
-#include "player.h"
+#include "log.h"
+#include "land.h"
+#include "ndata.h"
+#include "nlua.h"
+#include "nlua_audio.h"
+#include "nlua_bkg.h"
+#include "nlua_camera.h"
+#include "nlua_evt.h"
+#include "nlua_hook.h"
+#include "nlua_music.h"
+#include "nlua_tex.h"
+#include "nlua_tk.h"
+#include "nluadef.h"
 #include "npc.h"
-
+#include "nxml.h"
+#include "nxml_lua.h"
+#include "player.h"
+#include "rng.h"
 
 #define XML_EVENT_ID          "Events" /**< XML document identifier */
 #define XML_EVENT_TAG         "event" /**< XML event tag. */
 
-#define EVENT_CHUNK           32 /**< Size to grow event data by. */
-
-
 #define EVENT_FLAG_UNIQUE     (1<<0) /**< Unique event. */
 
-
 /**
  * @brief Event data structure.
  */
@@ -59,64 +55,64 @@ typedef struct EventData_ {
    char *name; /**< Name of the event. */
    char *sourcefile; /**< Source file code. */
    char *lua; /**< Lua code. */
+   int chunk; /**< Lua chunk. */
    unsigned int flags; /**< Bit flags. */
 
+   /* For specific cases. */
+   char *spob; /**< Spob name. */
+   char *system; /**< System name. */
+   char *chapter; /**< Chapter name. */
+   int *factions; /**< Faction checks. */
+   pcre2_code *chapter_re; /**< Compiled regex chapter if applicable. */
+
    EventTrigger_t trigger; /**< What triggers the event. */
    char *cond; /**< Conditional Lua code to execute. */
+   int cond_chunk; /**< Chunk of the conditional Lua code. */
    double chance; /**< Chance of appearing. */
    int priority; /**< Event priority: 0 = main plot, 5 = default, 10 = insignificant. */
-} EventData;
 
+   char **tags; /**< Tags. */
+} EventData;
 
 /*
  * Event data.
  */
 static EventData *event_data   = NULL; /**< Allocated event data. */
 
-
 /*
  * Active events.
  */
 static unsigned int event_genid  = 0; /**< Event ID generator. */
 static Event_t *event_active     = NULL; /**< Active events. */
-static int event_nactive         = 0; /**< Number of active events. */
-static int event_mactive         = 0; /**< Allocated space for active events. */
-
 
 /*
  * Prototypes.
  */
 static unsigned int event_genID (void);
 static int event_cmp( const void* a, const void* b );
-static int event_parseFile( const char* file );
+static int event_parseFile( const char* file, EventData *temp );
 static int event_parseXML( EventData *temp, const xmlNodePtr parent );
 static void event_freeData( EventData *event );
 static int event_create( int dataid, unsigned int *id );
 int events_saveActive( xmlTextWriterPtr writer );
-int events_loadActive( xmlNodePtr parent );;
+int events_loadActive( xmlNodePtr parent );
 static int events_parseActive( xmlNodePtr parent );
 
-
 /**
  * @brief Gets an event.
  */
 Event_t *event_get( unsigned int eventid )
 {
-   int i;
-   Event_t *ev;
-
    /* Iterate. */
-   for (i=0; i<event_nactive; i++) {
-      ev = &event_active[i];
+   for (int i=0; i<array_size(event_active); i++) {
+      Event_t *ev = &event_active[i];
       if (ev->id == eventid)
          return ev;
    }
 
-   /*WARN( "Event '%u' not found in stack.", eventid );*/
    return NULL;
 }
 
-
 /**
  * @brief Starts an event.
  *
@@ -140,56 +136,6 @@ int event_start( const char *name, unsigned int *id )
    return ret;
 }
 
-
-/**
- * @brief Starts running a function, allows programmer to set up arguments.
- */
-void event_runStart( unsigned int eventid, const char *func )
-{
-   Event_t *ev;
-
-   ev = event_get( eventid );
-   if (ev == NULL)
-      return;
-
-   event_setupLua( ev, func );
-}
-
-
-/**
- * @brief Runs a function previously set up with event_runStart.
- */
-int event_runFunc( unsigned int eventid, const char *func, int nargs )
-{
-   Event_t *ev;
-
-   ev = event_get( eventid );
-   if (ev == NULL)
-      return 0;
-
-   return event_runLuaFunc( ev, func, nargs );
-}
-
-
-/**
- * @brief Runs the event function.
- *
- *    @param eventid ID of the event to run Lua function on.
- *    @param func Name of the function to run.
- *    @return 0 on success.
- */
-int event_run( unsigned int eventid, const char *func )
-{
-   Event_t *ev;
-
-   ev = event_get( eventid );
-   if (ev == NULL)
-      return -1;
-
-   return event_runLua( ev, func );
-}
-
-
 /**
  * @brief Gets the name of the event data.
  *
@@ -198,16 +144,12 @@ int event_run( unsigned int eventid, const char *func )
  */
 const char *event_getData( unsigned int eventid )
 {
-   Event_t *ev;
-
-   ev = event_get( eventid );
+   const Event_t *ev = event_get( eventid );
    if (ev == NULL)
       return NULL;
-
    return event_data[ ev->data ].name;
 }
 
-
 /**
  * @brief Checks to see if an event is unique.
  *
@@ -216,16 +158,12 @@ const char *event_getData( unsigned int eventid )
  */
 int event_isUnique( unsigned int eventid )
 {
-   Event_t *ev;
-
-   ev = event_get( eventid );
+   const Event_t *ev = event_get( eventid );
    if (ev == NULL)
       return -1;
-
    return !!(event_data[ ev->data ].flags & EVENT_FLAG_UNIQUE);
 }
 
-
 /**
  * @brief Generates a new event ID.
  */
@@ -238,7 +176,6 @@ static unsigned int event_genID (void)
    return id;
 }
 
-
 /**
  * @brief Creates an event.
  *
@@ -250,26 +187,26 @@ static int event_create( int dataid, unsigned int *id )
 {
    Event_t *ev;
    EventData *data;
+   unsigned int eid;
+
+   if (event_active==NULL)
+      event_active = array_create( Event_t );
 
    /* Create the event. */
-   event_nactive++;
-   if (event_nactive > event_mactive) {
-      event_mactive += EVENT_CHUNK;
-      event_active = realloc( event_active, sizeof(Event_t) * event_mactive );
-   }
-   ev = &event_active[ event_nactive-1 ];
+   ev = &array_grow( &event_active );
    memset( ev, 0, sizeof(Event_t) );
    if ((id != NULL) && (*id != 0))
-      ev->id = *id;
+      eid = *id;
    else
-      ev->id = event_genID();
+      eid = event_genID();
+   ev->id = eid;
 
    /* Add the data. */
    ev->data = dataid;
    data = &event_data[dataid];
 
    /* Open the new state. */
-   ev->env = nlua_newEnv(1);
+   ev->env = nlua_newEnv();
    nlua_loadStandard(ev->env);
    nlua_loadEvt(ev->env);
    nlua_loadHook(ev->env);
@@ -277,11 +214,14 @@ static int event_create( int dataid, unsigned int *id )
    nlua_loadTex(ev->env);
    nlua_loadBackground(ev->env);
    nlua_loadMusic(ev->env);
-   nlua_loadSpfx(ev->env);
    nlua_loadTk(ev->env);
 
+   /* Create the "mem" table for persistence. */
+   lua_newtable(naevL);
+   nlua_setenv(naevL, ev->env, "mem");
+
    /* Load file. */
-   if (nlua_dobufenv(ev->env, data->lua, strlen(data->lua), data->sourcefile) != 0) {
+   if (nlua_dochunkenv(ev->env, data->chunk, data->sourcefile) != 0) {
       WARN(_("Error loading event file: %s\n"
             "%s\n"
             "Most likely Lua file has improper syntax, please check"),
@@ -291,14 +231,14 @@ static int event_create( int dataid, unsigned int *id )
 
    /* Run Lua. */
    if ((id==NULL) || (*id==0))
-      event_runLua( ev, "create" );
+      event_run( ev->id, "create" );
    if (id != NULL)
-      *id = ev->id;
+      *id = eid; /* The ev pointer can change and be invalidated in event_run,
+                    so we pass eid instead of ev->id. */
 
    return 0;
 }
 
-
 /**
  * @brief Cleans up an event.
  *
@@ -320,7 +260,6 @@ static void event_cleanup( Event_t *ev )
       claim_destroy( ev->claims );
 }
 
-
 /**
  * @brief Removes an event by ID.
  *
@@ -328,20 +267,15 @@ static void event_cleanup( Event_t *ev )
  */
 void event_remove( unsigned int eventid )
 {
-   int i;
-   Event_t *ev;
-
    /* Find the event. */
-   for (i=0; i<event_nactive; i++) {
-      ev = &event_active[i];
+   for (int i=0; i<array_size(event_active); i++) {
+      Event_t *ev = &event_active[i];
       if (ev->id == eventid) {
          /* Clean up event. */
          event_cleanup(ev);
 
          /* Move memory. */
-         memmove( &event_active[i], &event_active[i+1],
-               sizeof(Event_t) * (event_nactive-i-1) );
-         event_nactive--;
+         array_erase( &event_active, &event_active[i], &event_active[i+1] );
          return;
       }
    }
@@ -349,20 +283,17 @@ void event_remove( unsigned int eventid )
    WARN(_("Event ID '%u' not valid."), eventid);
 }
 
-
 /**
  * @brief Checks to see if an event should be saved.
  */
 int event_save( unsigned int eventid )
 {
-   Event_t *ev;
-   ev = event_get(eventid);
+   const Event_t *ev = event_get(eventid);
    if (ev == NULL)
       return 0;
    return ev->save;
 }
 
-
 /**
  * @brief Check to see if an event is already running.
  *
@@ -370,12 +301,9 @@ int event_save( unsigned int eventid )
  */
 int event_alreadyRunning( int data )
 {
-   int i;
-   Event_t *ev;
-
    /* Find events. */
-   for (i=0; i<event_nactive; i++) {
-      ev = &event_active[i];
+   for (int i=0; i<array_size(event_active); i++) {
+      const Event_t *ev = &event_active[i];
       if (ev->data == data)
          return 1;
    }
@@ -383,7 +311,6 @@ int event_alreadyRunning( int data )
    return 0;
 }
 
-
 /**
  * @brief Runs all the events matching a trigger.
  *
@@ -391,29 +318,82 @@ int event_alreadyRunning( int data )
  */
 void events_trigger( EventTrigger_t trigger )
 {
-   int i, c;
-   int created;
+   int created = 0;
+   for (int i=0; i<array_size(event_data); i++) {
+      EventData *ed = &event_data[i];
+
+      if (naev_isQuit())
+         return;
 
-   created = 0;
-   for (i=0; i<array_size(event_data); i++) {
       /* Make sure trigger matches. */
-      if (event_data[i].trigger != trigger)
+      if (ed->trigger != trigger)
+         continue;
+
+      /* Spob. */
+      if ((trigger==EVENT_TRIGGER_LAND || trigger==EVENT_TRIGGER_LOAD) && (ed->spob != NULL) && (strcmp(ed->spob,land_spob->name)!=0))
+         continue;
+
+      /* System. */
+      if ((ed->system != NULL) && (strcmp(ed->system,cur_system->name)!=0))
          continue;
 
       /* Make sure chance is succeeded. */
-      if (RNGF() > event_data[i].chance)
+      if (RNGF() > ed->chance)
          continue;
 
       /* Test uniqueness. */
-      if ((event_data[i].flags & EVENT_FLAG_UNIQUE) &&
-            (player_eventAlreadyDone( i ) || event_alreadyRunning(i)))
+      if ((ed->flags & EVENT_FLAG_UNIQUE) &&
+            (player_eventAlreadyDone(i) || event_alreadyRunning(i)))
          continue;
 
+      /* Test factions. */
+      if (ed->factions != NULL) {
+         int fct, match = 0;
+         if (trigger==EVENT_TRIGGER_ENTER)
+            fct = cur_system->faction;
+         else if (trigger==EVENT_TRIGGER_LOAD || trigger==EVENT_TRIGGER_LAND)
+            fct = land_spob->presence.faction;
+         else {
+            fct = -1;
+            match = -1; /* Don't hae to check factions. */
+         }
+
+         if (match==0) {
+            for (int j=0; j<array_size(ed->factions); j++) {
+               if (fct == ed->factions[j]) {
+                  match = 1;
+                  break;
+               }
+            }
+            if (!match)
+               continue;
+         }
+      }
+
+      /* If chapter, must match chapter regex. */
+      if (ed->chapter_re != NULL) {
+         pcre2_match_data *match_data = pcre2_match_data_create_from_pattern( ed->chapter_re, NULL );
+         int rc = pcre2_match( ed->chapter_re, (PCRE2_SPTR)player.chapter, strlen(player.chapter), 0, 0, match_data, NULL );
+         pcre2_match_data_free( match_data );
+         if (rc < 0) {
+            switch (rc) {
+               case PCRE2_ERROR_NOMATCH:
+                  continue;
+               default:
+                  WARN(_("Matching error %d"), rc );
+                  break;
+            }
+            continue;
+         }
+         else if (rc == 0)
+            continue;
+      }
+
       /* Test conditional. */
-      if (event_data[i].cond != NULL) {
-         c = cond_check(event_data[i].cond);
+      if (ed->cond != NULL) {
+         int c = cond_checkChunk( ed->cond_chunk, ed->cond );
          if (c<0) {
-            WARN(_("Conditional for event '%s' failed to run."), event_data[i].name);
+            WARN(_("Conditional for event '%s' failed to run."), ed->name);
             continue;
          }
          else if (!c)
@@ -430,7 +410,6 @@ void events_trigger( EventTrigger_t trigger )
       claim_activateAll();
 }
 
-
 /**
  * @brief Loads up an event from an XML node.
  *
@@ -440,26 +419,43 @@ void events_trigger( EventTrigger_t trigger )
  */
 static int event_parseXML( EventData *temp, const xmlNodePtr parent )
 {
-   xmlNodePtr node, cur;
-   char *buf;
+   xmlNodePtr node;
 
    memset( temp, 0, sizeof(EventData) );
 
+   /* Defaults. */
+   temp->chunk = LUA_NOREF;
+   temp->trigger = EVENT_TRIGGER_NULL;
+   temp->cond_chunk = LUA_NOREF;
+
    /* get the name */
-   temp->name = xml_nodeProp(parent, "name");
+   xmlr_attr_strd(parent, "name", temp->name);
    if (temp->name == NULL)
       WARN(_("Event in %s has invalid or no name"), EVENT_DATA_PATH);
 
    node = parent->xmlChildrenNode;
-
    do { /* load all the data */
-
       /* Only check nodes. */
       xml_onlyNodes(node);
 
+      xmlr_strd(node,"spob",temp->spob);
+      xmlr_strd(node,"system",temp->system);
+      xmlr_strd(node,"chapter",temp->chapter);
+
+      xmlr_strd(node,"cond",temp->cond);
+      xmlr_float(node,"chance",temp->chance);
+      xmlr_int(node,"priority",temp->priority);
+
+      if (xml_isNode(node,"faction")) {
+         if (temp->factions == NULL)
+            temp->factions = array_create( int );
+         array_push_back( &temp->factions, faction_get( xml_get(node) ) );
+         continue;
+      }
+
       /* Trigger. */
-      if (xml_isNode(node,"trigger")) {
-         buf = xml_get(node);
+      if (xml_isNode(node,"location")) {
+         char *buf = xml_get(node);
          if (buf == NULL)
             WARN(_("Event '%s': Null trigger type."), temp->name);
          else if (strcmp(buf,"enter")==0)
@@ -477,47 +473,66 @@ static int event_parseXML( EventData *temp, const xmlNodePtr parent )
       }
 
       /* Flags. */
-      else if (xml_isNode(node,"flags")) { /* set the various flags */
-         cur = node->children;
+      else if (xml_isNode(node,"unique")) { /* unique event. */
+         temp->flags |= EVENT_FLAG_UNIQUE;
+         continue;
+      }
+
+      /* Tags. */
+      if (xml_isNode(node,"tags")) {
+         xmlNodePtr cur = node->children;
+         temp->tags = array_create( char* );
          do {
             xml_onlyNodes(cur);
-            if (xml_isNode(cur,"unique")) {
-               temp->flags |= EVENT_FLAG_UNIQUE;
+            if (xml_isNode(cur, "tag")) {
+               const char *tmp = xml_get(cur);
+               if (tmp != NULL)
+                  array_push_back( &temp->tags, strdup(tmp) );
                continue;
             }
-            WARN(_("Event '%s' has unknown flag node '%s'."), temp->name, cur->name);
+            WARN(_("Event '%s' has unknown node in tags '%s'."), temp->name, cur->name );
          } while (xml_nextNode(cur));
          continue;
       }
 
       /* Notes for the python mission mapping script. */
-      else if (xml_isNode(node,"notes")) continue;
+      else if (xml_isNode(node,"notes"))
+         continue;
 
-      /* Condition. */
-      xmlr_strd(node,"cond",temp->cond);
-
-      /* Get chance. */
-      xmlr_float(node,"chance",temp->chance);
-
-      /* Get proirity. */
-      xmlr_int(node,"priority",temp->priority);
-
-      DEBUG(_("Unknown node '%s' in event '%s'"), node->name, temp->name);
+      WARN(_("Unknown node '%s' in event '%s'"), node->name, temp->name);
    } while (xml_nextNode(node));
 
    /* Process. */
    temp->chance /= 100.;
 
+   /* Compile conditional chunk. */
+   if (temp->cond != NULL) {
+      temp->cond_chunk = cond_compile( temp->cond );
+      if (temp->cond_chunk == LUA_NOREF || temp->cond_chunk == LUA_REFNIL)
+         WARN(_("Event '%s' failed to compile Lua conditional!"), temp->name);
+   }
+
+   /* Compile regex for chapter matching. */
+   if (temp->chapter != NULL) {
+      int errornumber;
+      PCRE2_SIZE erroroffset;
+      temp->chapter_re = pcre2_compile( (PCRE2_SPTR)temp->chapter, PCRE2_ZERO_TERMINATED, 0, &errornumber, &erroroffset, NULL );
+      if (temp->chapter_re == NULL){
+         PCRE2_UCHAR buffer[256];
+         pcre2_get_error_message( errornumber, buffer, sizeof(buffer) );
+         WARN(_("Mission '%s' chapter PCRE2 compilation failed at offset %d: %s"), temp->name, (int)erroroffset, buffer );
+      }
+   }
+
 #define MELEMENT(o,s) \
    if (o) WARN(_("Event '%s' missing/invalid '%s' element"), temp->name, s)
+   MELEMENT(temp->trigger==EVENT_TRIGGER_NULL,"location");
    MELEMENT((temp->trigger!=EVENT_TRIGGER_NONE) && (temp->chance==0.),"chance");
-   MELEMENT(temp->trigger==EVENT_TRIGGER_NULL,"trigger");
 #undef MELEMENT
 
    return 0;
 }
 
-
 static int event_cmp( const void* a, const void* b )
 {
    const EventData *ea, *eb;
@@ -530,7 +545,6 @@ static int event_cmp( const void* a, const void* b )
    return strcmp( ea->name, eb->name );
 }
 
-
 /**
  * @brief Loads all the events.
  *
@@ -538,44 +552,58 @@ static int event_cmp( const void* a, const void* b )
  */
 int events_load (void)
 {
-   int    i;
-   char **event_files;
+#if DEBUGGING
+   Uint32 time = SDL_GetTicks();
+#endif /* DEBUGGING */
+   char **event_files = ndata_listRecursive( EVENT_DATA_PATH );
 
    /* Run over events. */
-   event_files = ndata_listRecursive( EVENT_DATA_PATH );
-   event_data  = array_create_size( EventData, array_size( event_files ) );
-   for ( i = 0; i < array_size( event_files ); i++ ) {
-      event_parseFile( event_files[ i ] );
+   event_data = array_create_size( EventData, array_size( event_files ) );
+   for (int i=0; i < array_size( event_files ); i++) {
+      event_parseFile( event_files[i], NULL );
       free( event_files[ i ] );
    }
    array_free( event_files );
    array_shrink( &event_data );
 
+#ifdef DEBUGGING
+   for (int i=0; i<array_size(event_data); i++) {
+      EventData *ed = &event_data[i];
+      for (int j=i+1; j<array_size(event_data); j++)
+         if (strcmp( ed->name, event_data[j].name )==0)
+            WARN(_("Duplicate event '%s'!"), ed->name);
+   }
+#endif /* DEBUGGING */
+
    /* Sort based on priority so higher priority missions can establish claims first. */
    qsort( event_data, array_size(event_data), sizeof(EventData), event_cmp );
 
-   DEBUG( ngettext("Loaded %d Event", "Loaded %d Events", array_size(event_data) ), array_size(event_data) );
+#if DEBUGGING
+   if (conf.devmode) {
+      time = SDL_GetTicks() - time;
+      DEBUG( n_("Loaded %d Event in %.3f s", "Loaded %d Events in %.3f s", array_size(event_data) ), array_size(event_data), time/1000. );
+   }
+   else
+      DEBUG( n_("Loaded %d Event", "Loaded %d Events", array_size(event_data) ), array_size(event_data) );
+#endif /* DEBUGGING */
 
    return 0;
 }
 
-
 /**
  * @brief Parses an event file.
+ *
+ *    @param file Source file path.
+ *    @param temp Data to load into, or NULL for initial load.
  */
-static int event_parseFile( const char* file )
+static int event_parseFile( const char* file, EventData *temp )
 {
    size_t bufsize;
    xmlNodePtr node;
    xmlDocPtr doc;
    char *filebuf;
    const char *pos, *start_pos;
-   EventData *temp;
-
-#ifdef DEBUGGING
-   /* To check if event is valid. */
    int ret;
-#endif /* DEBUGGING */
 
    /* Load string. */
    filebuf = ndata_read( file, &bufsize );
@@ -583,11 +611,15 @@ static int event_parseFile( const char* file )
       WARN(_("Unable to read data from '%s'"), file);
       return -1;
    }
+   if (bufsize == 0) {
+      free( filebuf );
+      return -1;
+   }
 
    /* Skip if no XML. */
-   pos = nstrnstr( filebuf, "</event>", bufsize );
+   pos = strnstr( filebuf, "</event>", bufsize );
    if (pos==NULL) {
-      pos = nstrnstr( filebuf, "function create", bufsize );
+      pos = strnstr( filebuf, "function create", bufsize );
       if ((pos != NULL) && !strncmp(pos,"--common",bufsize))
          WARN(_("Event '%s' has create function but no XML header!"), file);
       free(filebuf);
@@ -595,8 +627,8 @@ static int event_parseFile( const char* file )
    }
 
    /* Separate XML header and Lua. */
-   start_pos = nstrnstr( filebuf, "<?xml ", bufsize );
-   pos = nstrnstr( filebuf, "--]]", bufsize );
+   start_pos = strnstr( filebuf, "<?xml ", bufsize );
+   pos = strnstr( filebuf, "--]]", bufsize );
    if (pos == NULL || start_pos == NULL) {
       WARN(_("Event file '%s' has missing XML header!"), file);
       return -1;
@@ -616,21 +648,24 @@ static int event_parseFile( const char* file )
       return -1;
    }
 
-   temp = &array_grow(&event_data);
+   if (temp == NULL)
+      temp = &array_grow(&event_data);
    event_parseXML( temp, node );
    temp->lua = strdup(filebuf);
    temp->sourcefile = strdup(file);
 
-#ifdef DEBUGGING
+   /* Clear chunk if already loaded. */
+   if (temp->chunk != LUA_NOREF) {
+      luaL_unref( naevL, LUA_REGISTRYINDEX, temp->chunk );
+      temp->chunk = LUA_NOREF;
+   }
+
    /* Check to see if syntax is valid. */
    ret = luaL_loadbuffer(naevL, temp->lua, strlen(temp->lua), temp->name );
-   if (ret == LUA_ERRSYNTAX) {
-      WARN(_("Event Lua '%s' syntax error: %s"),
-            file, lua_tostring(naevL,-1) );
-   } else {
-      lua_pop(naevL, 1);
-   }
-#endif /* DEBUGGING */
+   if (ret == LUA_ERRSYNTAX)
+      WARN(_("Event Lua '%s' syntax error: %s"), file, lua_tostring(naevL,-1) );
+   else
+      temp->chunk = luaL_ref( naevL, LUA_REGISTRYINDEX );
 
    /* Clean up. */
    xmlFreeDoc(doc);
@@ -639,7 +674,6 @@ static int event_parseFile( const char* file )
    return 0;
 }
 
-
 /**
  * @brief Frees an EventData structure.
  *
@@ -647,58 +681,61 @@ static int event_parseFile( const char* file )
  */
 static void event_freeData( EventData *event )
 {
-   if (event->name)
-      free( event->name );
-   if (event->lua)
-      free( event->lua );
-   if (event->sourcefile)
-      free( event->sourcefile );
-   if (event->cond)
-      free( event->cond );
+   free( event->name );
+   free( event->sourcefile );
+   free( event->lua );
+
+   free( event->spob );
+   free( event->system );
+   free( event->chapter );
+   pcre2_code_free( event->chapter_re );
+
+   array_free( event->factions );
+
+   free( event->cond );
+
+   if (event->chunk != LUA_NOREF)
+      luaL_unref( naevL, LUA_REGISTRYINDEX,event->chunk );
+
+   if (event->cond_chunk != LUA_NOREF)
+      luaL_unref( naevL, LUA_REGISTRYINDEX,event->cond_chunk );
+
+   for (int i=0; i<array_size(event->tags); i++)
+      free(event->tags[i]);
+   array_free(event->tags);
+
+   /* Clear the memory. */
 #if DEBUGGING
    memset( event, 0, sizeof(EventData) );
 #endif /* DEBUGGING */
 }
 
-
 /**
  * @brief Cleans up and removes active events.
  */
 void events_cleanup (void)
 {
-   int i;
-
    /* Free active events. */
-   for (i=0; i<event_nactive; i++)
+   for (int i=0; i<array_size(event_active); i++)
       event_cleanup( &event_active[i] );
-   if (event_active != NULL)
-      free(event_active);
-
+   array_free(event_active);
    event_active = NULL;
-   event_nactive = 0;
-   event_mactive = 0;
 }
 
-
 /**
  * @brief Exits the event subsystem.
  */
 void events_exit (void)
 {
-   int i;
-
    events_cleanup();
 
    /* Free data. */
-   if (event_data != NULL) {
-      for (i=0; i<array_size(event_data); i++)
-         event_freeData( &event_data[i] );
-      array_free(event_data);
-   }
+   for (int i=0; i<array_size(event_data); i++)
+      event_freeData( &event_data[i] );
+   array_free(event_data);
    event_data  = NULL;
 }
 
-
 /**
  * @brief Gets the event data id from name.
  *
@@ -707,16 +744,13 @@ void events_exit (void)
  */
 int event_dataID( const char *evdata )
 {
-   int i;
-
-   for (i=0; i<array_size(event_data); i++)
+   for (int i=0; i<array_size(event_data); i++)
       if (strcmp(event_data[i].name, evdata)==0)
          return i;
    WARN(_("No event data found matching name '%s'."), evdata);
    return -1;
 }
 
-
 /**
  * @brief Gets the event data name from id.
  *
@@ -728,43 +762,38 @@ const char *event_dataName( int dataid )
    return event_data[dataid].name;
 }
 
-
 /**
  * @brief Activates all the active event claims.
  */
 void event_activateClaims (void)
 {
-   int i;
-
    /* Free active events. */
-   for (i=0; i<event_nactive; i++)
+   for (int i=0; i<array_size(event_active); i++)
       if (event_active[i].claims != NULL)
          claim_activate( event_active[i].claims );
 }
 
-
 /**
  * @brief Tests to see if an event has claimed a system.
  */
 int event_testClaims( unsigned int eventid, int sys )
 {
-   Event_t *ev;
-   ev = event_get( eventid );
+   const Event_t *ev = event_get( eventid );
+   if (ev==NULL) {
+      WARN(_("Trying to test claims of unknown event with id '%d'!"), eventid);
+      return 0;
+   }
    return claim_testSys( ev->claims, sys );
 }
 
-
 /**
  * @brief Checks the event validity and cleans up after them.
  */
 void event_checkValidity (void)
 {
-   int i;
-   Event_t *ev;
-
    /* Iterate. */
-   for (i=0; i<event_nactive; i++) {
-      ev = &event_active[i];
+   for (int i=0; i<array_size(event_active); i++) {
+      const Event_t *ev = &event_active[i];
 
       /* Check if has children. */
       if (hook_hasEventParent( ev->id ) > 0)
@@ -778,7 +807,6 @@ void event_checkValidity (void)
    }
 }
 
-
 /**
  * @brief Saves the player's active events.
  *
@@ -787,13 +815,10 @@ void event_checkValidity (void)
  */
 int events_saveActive( xmlTextWriterPtr writer )
 {
-   int i;
-   Event_t *ev;
-
    xmlw_startElem(writer,"events");
 
-   for (i=0; i<event_nactive; i++) {
-      ev = &event_active[i];
+   for (int i=0; i<array_size(event_active); i++) {
+      Event_t *ev = &event_active[i];
       if (!ev->save) /* Only save events that want to be saved. */
          continue;
 
@@ -820,7 +845,6 @@ int events_saveActive( xmlTextWriterPtr writer )
    return 0;
 }
 
-
 /**
  * @brief Loads the player's active events from a save.
  *
@@ -829,21 +853,22 @@ int events_saveActive( xmlTextWriterPtr writer )
  */
 int events_loadActive( xmlNodePtr parent )
 {
-   xmlNodePtr node;
+   int failed = 0;
+   xmlNodePtr node = parent->xmlChildrenNode;
 
    /* cleanup old events */
    events_cleanup();
 
-   node = parent->xmlChildrenNode;
    do {
-      if (xml_isNode(node,"events"))
-         if (events_parseActive( node ) < 0) return -1;
+      if (xml_isNode(node,"events")) {
+         int ret = events_parseActive( node );
+         failed |= ret;
+      }
    } while (xml_nextNode(node));
 
-   return 0;
+   return failed;
 }
 
-
 /**
  * @brief Parses the actual individual event nodes.
  *
@@ -852,18 +877,18 @@ int events_loadActive( xmlNodePtr parent )
  */
 static int events_parseActive( xmlNodePtr parent )
 {
-   char *buf;
-   unsigned int id;
-   int data;
-   xmlNodePtr node, cur;
-   Event_t *ev;
-
-   node = parent->xmlChildrenNode;
+   xmlNodePtr node = parent->xmlChildrenNode;
    do {
+      char *buf;
+      unsigned int id;
+      int data;
+      xmlNodePtr cur;
+      Event_t *ev;
+
       if (!xml_isNode(node,"event"))
          continue;
 
-      xmlr_attr(node,"name",buf);
+      xmlr_attr_strd(node,"name",buf);
       if (buf==NULL) {
          WARN(_("Event has missing 'name' attribute, skipping."));
          continue;
@@ -875,13 +900,7 @@ static int events_parseActive( xmlNodePtr parent )
          continue;
       }
       free(buf);
-      xmlr_attr(node,"id",buf);
-      if (buf==NULL) {
-         WARN(_("Event with data '%s' has missing 'id' attribute, skipping."), event_dataName(data));
-         continue;
-      }
-      id = atoi(buf);
-      free(buf);
+      xmlr_attr_uint(node,"id",id);
       if (id==0) {
          WARN(_("Event with data '%s' has invalid 'id' attribute, skipping."), event_dataName(data));
          continue;
@@ -899,8 +918,14 @@ static int events_parseActive( xmlNodePtr parent )
       /* Get the data. */
       cur = node->xmlChildrenNode;
       do {
-         if (xml_isNode(cur,"lua"))
-            nxml_unpersistLua( ev->env, cur );
+         if (xml_isNode(cur,"lua")) {
+            int ret = nxml_unpersistLua( ev->env, cur );
+            if (ret) {
+               WARN(_("Event with data '%s' failed to unpersist Lua, skipping."), event_dataName(data));
+               event_remove( id );
+               return -1;
+            }
+         }
       } while (xml_nextNode(cur));
 
       /* Claims. */
@@ -911,4 +936,37 @@ static int events_parseActive( xmlNodePtr parent )
    return 0;
 }
 
+int event_reload( const char *name )
+{
+   int res, edat = event_dataID( name );
+   EventData save, *temp = edat<0 ? NULL : &event_data[edat];
+   if (temp == NULL)
+      return -1;
+   save = *temp;
+   res = event_parseFile( save.sourcefile, temp );
+   if (res == 0)
+      event_freeData( &save );
+   else
+      *temp = save;
+   return res;
+}
 
+void event_toLuaTable( lua_State *L, int eventid )
+{
+   const EventData *data = &event_data[ eventid ];
+
+   lua_newtable(L);
+
+   lua_pushstring(L, data->name);
+   lua_setfield(L,-2,"name");
+
+   lua_pushboolean(L, (data->flags & EVENT_FLAG_UNIQUE));
+   lua_setfield(L,-2,"unique");
+
+   lua_newtable(L);
+   for (int j=0; j<array_size(data->tags); j++) {
+      lua_pushboolean(L,1);
+      lua_setfield(L,-2,data->tags[j]);
+   }
+   lua_setfield(L,-2,"tags");
+}
