@@ -1,21 +1,16 @@
 /*
  * See Licensing and Copyright notice in naev.h
  */
+#pragma once
 
-
-
-#ifndef GUI_OMSG_H
-#  define GUI_OMSG_H
-
-
+#include "colour.h"
 
 #define OMSG_FONT_DEFAULT_SIZE      16
 
-
 /*
  * Creation and management.
  */
-unsigned int omsg_add( const char *msg, double duration, int fontsize );
+unsigned int omsg_add( const char *msg, double duration, int fontsize, const glColour *col );
 int omsg_change( unsigned int id, const char *msg, double duration );
 int omsg_exists( unsigned int id );
 void omsg_rm( unsigned int id );
@@ -26,6 +21,3 @@ void omsg_rm( unsigned int id );
 void omsg_position( double center_x, double center_y, double width );
 void omsg_cleanup (void);
 void omsg_render( double dt );
-
-
-#endif /* GUI_OMSG_H */
