@@ -1,33 +1,36 @@
 /*
  * See Licensing and Copyright notice in naev.h
  */
-
 /**
  * @file nlua_system.c
  *
  * @brief Lua system module.
  */
+/** @cond */
+#include <lauxlib.h>
+
+#include "naev.h"
+/** @endcond */
 
 #include "nlua_system.h"
 
-#include "naev.h"
-
-#include <lauxlib.h>
-
-#include "nluadef.h"
-#include "nlua_faction.h"
-#include "nlua_vec2.h"
-#include "nlua_planet.h"
-#include "nlua_jump.h"
-#include "log.h"
-#include "rng.h"
+#include "array.h"
 #include "land.h"
 #include "land_outfits.h"
+#include "log.h"
+#include "gatherable.h"
 #include "map.h"
 #include "map_overlay.h"
+#include "nebula.h"
+#include "nlua_commodity.h"
+#include "nlua_faction.h"
+#include "nlua_jump.h"
+#include "nlua_spob.h"
+#include "nlua_vec2.h"
+#include "nluadef.h"
+#include "rng.h"
 #include "space.h"
 
-
 /* System metatable methods */
 static int systemL_cur( lua_State *L );
 static int systemL_get( lua_State *L );
@@ -35,25 +38,29 @@ static int systemL_getAll( lua_State *L );
 static int systemL_eq( lua_State *L );
 static int systemL_name( lua_State *L );
 static int systemL_nameRaw( lua_State *L );
+static int systemL_position( lua_State *L );
 static int systemL_faction( lua_State *L );
+static int systemL_background( lua_State *L );
 static int systemL_nebula( lua_State *L );
+static int systemL_interference( lua_State *L );
 static int systemL_jumpdistance( lua_State *L );
 static int systemL_jumpPath( lua_State *L );
 static int systemL_adjacent( lua_State *L );
 static int systemL_jumps( lua_State *L );
-static int systemL_asteroid( lua_State *L );
-static int systemL_asteroidPos( lua_State *L );
-static int systemL_asteroidDestroyed( lua_State *L );
+static int systemL_asteroidFields( lua_State *L );
 static int systemL_addGatherable( lua_State *L );
 static int systemL_presences( lua_State *L );
-static int systemL_planets( lua_State *L );
+static int systemL_spobs( lua_State *L );
 static int systemL_presence( lua_State *L );
 static int systemL_radius( lua_State *L );
 static int systemL_isknown( lua_State *L );
 static int systemL_setknown( lua_State *L );
-static int systemL_mrkClear( lua_State *L );
-static int systemL_mrkAdd( lua_State *L );
-static int systemL_mrkRm( lua_State *L );
+static int systemL_hidden( lua_State *L );
+static int systemL_setHidden( lua_State *L );
+static int systemL_markerClear( lua_State *L );
+static int systemL_markerAdd( lua_State *L );
+static int systemL_markerRm( lua_State *L );
+static int systemL_tags( lua_State *L );
 static const luaL_Reg system_methods[] = {
    { "cur", systemL_cur },
    { "get", systemL_get },
@@ -62,29 +69,32 @@ static const luaL_Reg system_methods[] = {
    { "__tostring", systemL_name },
    { "name", systemL_name },
    { "nameRaw", systemL_nameRaw },
+   { "pos", systemL_position },
    { "faction", systemL_faction },
+   { "background", systemL_background },
    { "nebula", systemL_nebula },
+   { "interference", systemL_interference },
    { "jumpDist", systemL_jumpdistance },
    { "jumpPath", systemL_jumpPath },
    { "adjacentSystems", systemL_adjacent },
    { "jumps", systemL_jumps },
-   { "asteroid", systemL_asteroid },
-   { "asteroidPos", systemL_asteroidPos },
-   { "asteroidDestroyed", systemL_asteroidDestroyed },
+   { "asteroidFields", systemL_asteroidFields },
    { "addGatherable", systemL_addGatherable },
    { "presences", systemL_presences },
-   { "planets", systemL_planets },
+   { "spobs", systemL_spobs },
    { "presence", systemL_presence },
    { "radius", systemL_radius },
    { "known", systemL_isknown },
    { "setKnown", systemL_setknown },
-   { "mrkClear", systemL_mrkClear },
-   { "mrkAdd", systemL_mrkAdd },
-   { "mrkRm", systemL_mrkRm },
+   { "hidden", systemL_hidden },
+   { "setHidden", systemL_setHidden },
+   { "markerClear", systemL_markerClear },
+   { "markerAdd", systemL_markerAdd },
+   { "markerRm", systemL_markerRm },
+   { "tags", systemL_tags },
    {0,0}
 }; /**< System metatable methods. */
 
-
 /**
  * @brief Loads the system library.
  *
@@ -97,7 +107,6 @@ int nlua_loadSystem( nlua_env env )
    return 0; /* No error */
 }
 
-
 /**
  * @brief Lua system module.
  *
@@ -105,7 +114,7 @@ int nlua_loadSystem( nlua_env env )
  *
  * Typical example would be something like:
  * @code
- * cur = system.get() -- Gets current system
+ * cur = system.cur() -- Gets current system
  * sys = system.get( "Gamma Polaris" )
  * @endcode
  *
@@ -146,11 +155,10 @@ LuaSystem luaL_checksystem( lua_State *L, int ind )
  */
 StarSystem* luaL_validsystem( lua_State *L, int ind )
 {
-   LuaSystem ls;
    StarSystem *s;
 
    if (lua_issystem(L, ind)) {
-      ls = luaL_checksystem(L, ind);
+      LuaSystem ls = luaL_checksystem(L, ind);
       s = system_getIndex( ls );
    }
    else if (lua_isstring(L, ind))
@@ -175,8 +183,7 @@ StarSystem* luaL_validsystem( lua_State *L, int ind )
  */
 LuaSystem* lua_pushsystem( lua_State *L, LuaSystem sys )
 {
-   LuaSystem *s;
-   s = (LuaSystem*) lua_newuserdata(L, sizeof(LuaSystem));
+   LuaSystem *s = (LuaSystem*) lua_newuserdata(L, sizeof(LuaSystem));
    *s = sys;
    luaL_getmetatable(L, SYSTEM_METATABLE);
    lua_setmetatable(L, -2);
@@ -206,14 +213,13 @@ int lua_issystem( lua_State *L, int ind )
    return ret;
 }
 
-
 /**
  * @brief Gets the current system.
  *
  * @usage sys = system.cur() -- Gets the current system
  *
  *    @luatreturn System Current system.
- * @luafunc cur()
+ * @luafunc cur
  */
 static int systemL_cur( lua_State *L )
 {
@@ -221,42 +227,44 @@ static int systemL_cur( lua_State *L )
    return 1;
 }
 
-
 /**
  * @brief Gets a system.
  *
  * Behaves differently depending on what you pass as param: <br/>
  *    - string : Gets the system by raw (untranslated) name. <br/>
- *    - planet : Gets the system by planet. <br/>
+ *    - spob : Gets the system by spob. <br/>
  *
- * @usage sys = system.get( p ) -- Gets system where planet 'p' is located.
+ * @usage sys = system.get( p ) -- Gets system where spob 'p' is located.
  * @usage sys = system.get( "Gamma Polaris" ) -- Gets the system by name.
  *
- *    @luatparam string|Planet param Read description for details.
+ *    @luatparam string|Spob param Read description for details.
  *    @luatreturn System System matching param.
- * @luafunc get( param )
+ * @luafunc get
  */
 static int systemL_get( lua_State *L )
 {
    StarSystem *ss;
-   Planet *pnt;
+   Spob *pnt;
 
    /* Passing a string (systemname) */
    if (lua_isstring(L,1)) {
       ss = system_get( lua_tostring(L,1) );
    }
-   /* Passing a planet */
-   else if (lua_isplanet(L,1)) {
-      pnt = luaL_validplanet(L,1);
-      ss = system_get( planet_getSystem( pnt->name ) );
+   /* Passing a spob */
+   else if (lua_isspob(L,1)) {
+      pnt = luaL_validspob(L,1);
+      ss = system_get( spob_getSystem( pnt->name ) );
    }
-   else NLUA_INVALID_PARAMETER(L);
+   else if (lua_issystem(L,1)) {
+      lua_pushvalue(L,1);
+      return 1;
+   }
+   else
+      NLUA_INVALID_PARAMETER(L,1);
 
    /* Error checking. */
-   if (ss == NULL) {
-      NLUA_ERROR(L, _("No matching systems found."));
-      return 0;
-   }
+   if (ss == NULL)
+      return NLUA_ERROR(L, _("No matching systems found."));
 
    /* return the system */
    lua_pushsystem(L,system_index(ss));
@@ -266,21 +274,15 @@ static int systemL_get( lua_State *L )
 /**
  * @brief Gets all the systems.
  *    @luatreturn {System,...} A list of all the systems.
- * @luafunc getAll()
+ * @luafunc getAll
  */
 static int systemL_getAll( lua_State *L )
 {
-   StarSystem *sys;
-   int i, ind, n;
-
+   StarSystem *sys = system_getAll();
    lua_newtable(L);
-   sys = system_getAll( &n );
-
-   ind = 1;
-   for (i=0; i<n; i++) {
-      lua_pushnumber( L, ind++ );
+   for (int i=0; i<array_size(sys); i++) {
       lua_pushsystem( L, system_index( &sys[i] ) );
-      lua_settable(   L, -3 );
+      lua_rawseti( L, -2, i+1 );
    }
    return 1;
 }
@@ -295,7 +297,7 @@ static int systemL_getAll( lua_State *L )
  *    @luatparam System s System comparing.
  *    @luatparam System comp System to compare against.
  *    @luatreturn boolean true if both systems are the same.
- * @luafunc __eq( s, comp )
+ * @luafunc __eq
  */
 static int systemL_eq( lua_State *L )
 {
@@ -320,16 +322,29 @@ static int systemL_eq( lua_State *L )
  *
  *    @luatparam System s System to get the translated name of.
  *    @luatreturn string The translated name of the system.
- * @luafunc name( s )
+ * @luafunc name
  */
 static int systemL_name( lua_State *L )
 {
-   StarSystem *sys;
-   sys = luaL_validsystem(L,1);
+   const StarSystem *sys = luaL_validsystem(L,1);
    lua_pushstring(L, _(sys->name));
    return 1;
 }
 
+/**
+ * @brief Returns the position of the system.
+ *
+ *    @luatparam System s System to get position of.
+ *    @luatreturn vec2 Position of the system.
+ * @luafunc pos
+ */
+static int systemL_position( lua_State *L )
+{
+   const StarSystem *sys = luaL_validsystem(L,1);
+   lua_pushvector(L, sys->pos);
+   return 1;
+}
+
 /**
  * @brief Returns the system's raw (untranslated) name.
  *
@@ -341,12 +356,11 @@ static int systemL_name( lua_State *L )
  *
  *    @luatparam System s System to get the raw name of.
  *    @luatreturn string The raw name of the system.
- * @luafunc nameRaw( s )
+ * @luafunc nameRaw
  */
 static int systemL_nameRaw( lua_State *L )
 {
-   StarSystem *sys;
-   sys = luaL_validsystem(L,1);
+   StarSystem *sys = luaL_validsystem(L,1);
    lua_pushstring(L, sys->name);
    return 1;
 }
@@ -356,46 +370,67 @@ static int systemL_nameRaw( lua_State *L )
  *
  *    @luatparam System s System to get the faction of.
  *    @luatreturn Faction The dominant faction in the system.
- * @luafunc faction( s )
+ * @luafunc faction
  */
 static int systemL_faction( lua_State *L )
 {
-   StarSystem *s;
-
-   s = luaL_validsystem(L,1);
-
+   const StarSystem *s = luaL_validsystem(L,1);
    if (s->faction == -1)
       return 0;
-
    lua_pushfaction(L,s->faction);
    return 1;
 
 }
 
+/**
+ * @brief Gets system background.
+ *
+ *    @luatparam System s System to get the background of.
+ *    @luatreturn string|nil The background of the system or nil for default.
+ * @luafunc background
+ */
+static int systemL_background( lua_State *L )
+{
+   StarSystem *s = luaL_validsystem(L,1);
+   if (s->background==NULL)
+      return 0;
+   lua_pushstring(L,s->background);
+   return 1;
+
+}
 
 /**
  * @brief Gets the system's nebula parameters.
  *
- * @usage density, volatility = sys:nebula()
+ * @usage density, volatility, damage = sys:nebula()
  *
  *    @luatparam System s System to get nebula parameters from.
  *    @luatreturn number The density of the system.
- *    @luatreturn number The volatility of the system.
- * @luafunc nebula( s )
+ *    @luatreturn number The amount of nebula damage done per second.
+ * @luafunc nebula
  */
 static int systemL_nebula( lua_State *L )
 {
-   StarSystem *s;
-
-   s = luaL_validsystem(L,1);
-
+   StarSystem *s = luaL_validsystem(L,1);
    /* Push the density and volatility. */
    lua_pushnumber(L, s->nebu_density);
    lua_pushnumber(L, s->nebu_volatility);
-
    return 2;
 }
 
+/**
+ * @brief Gets the system's interference level.
+ *
+ *    @luatparam System s System to get interference of.
+ *    @luatreturn number The amount of interference (in percent).
+ * @luafunc interference
+ */
+static int systemL_interference( lua_State *L )
+{
+   StarSystem *s = luaL_validsystem(L,1);
+   lua_pushnumber( L, s->interference );
+   return 1;
+}
 
 /**
  * @brief Gets jump distance from current system, or to another.
@@ -405,7 +440,7 @@ static int systemL_nebula( lua_State *L )
  *    - string : Gets distance to system matching name.
  *    - system : Gets distance to system
  *
- * @usage d = sys:jumpDist() -- Distance from sys to current system.
+ * @usage d = sys:jumpDist() -- Distance the current system to sys.
  * @usage d = sys:jumpDist( "Draygar" ) -- Distance from sys to system Draygar.
  * @usage d = sys:jumpDist( another_sys ) -- Distance from sys to another_sys.
  *
@@ -413,14 +448,13 @@ static int systemL_nebula( lua_State *L )
  *    @luatparam nil|string|Goal system param See description.
  *    @luatparam[opt=false] boolean hidden Whether or not to consider hidden jumps.
  *    @luatparam[opt=false] boolean known Whether or not to consider only jumps known by the player.
- *    @luatreturn number Number of jumps to system.
- * @luafunc jumpDist( s, param, hidden, known )
+ *    @luatreturn number Number of jumps to system or math.huge if no path found.
+ * @luafunc jumpDist
  */
 static int systemL_jumpdistance( lua_State *L )
 {
-   StarSystem *sys, *sysp;
+   StarSystem *sys;
    StarSystem **s;
-   int jumps;
    const char *start, *goal;
    int h, k;
 
@@ -433,22 +467,34 @@ static int systemL_jumpdistance( lua_State *L )
       if (lua_isstring(L,2))
          goal = lua_tostring(L,2);
       else if (lua_issystem(L,2)) {
-         sysp = luaL_validsystem(L,2);
+         const StarSystem *sysp = luaL_validsystem(L,2);
          goal = sysp->name;
       }
-      else NLUA_INVALID_PARAMETER(L);
+      else
+         NLUA_INVALID_PARAMETER(L,2);
+   }
+   else {
+      goal  = sys->name;
+      start = cur_system->name;
    }
-   else
-      goal = cur_system->name;
 
-   s = map_getJumpPath( &jumps, start, goal, k, h, NULL );
-   free(s);
+   /* Trivial case same system. */
+   if (strcmp(start,goal)==0) {
+      lua_pushnumber(L, 0.);
+      return 1;
+   }
 
-   lua_pushnumber(L,jumps);
+   s = map_getJumpPath( start, NULL, goal, k, h, NULL, NULL );
+   if (s==NULL) {
+      lua_pushnumber(L, HUGE_VAL);
+      return 1;
+   }
+
+   lua_pushnumber(L,array_size(s));
+   array_free(s);
    return 1;
 }
 
-
 /**
  * @brief Gets jump path from current system, or to another.
  *
@@ -459,49 +505,34 @@ static int systemL_jumpdistance( lua_State *L )
  *    <li>system : Gets path to system</li>
  * </ul>
  *
- * @usage jumps = sys:jumpPath() -- Path from sys to current system.
+ * @usage jumps = sys:jumpPath( system.cur() ) -- Path from sys to current system.
  * @usage jumps = sys:jumpPath( "Draygar" ) -- Path from sys to Draygar.
  * @usage jumps = system.jumpPath( "Draygar", another_sys ) -- Path from Draygar to another_sys.
  *
  *    @luatparam System s Starting system.
- *    @luatparam nil|string|Goal system param See description.
+ *    @luatparam System goal Goal system param See description.
  *    @luatparam[opt=false] boolean hidden Whether or not to consider hidden jumps.
  *    @luatreturn {Jump,...} Table of jumps.
- * @luafunc jumpPath( s, param, hidden )
+ * @luafunc jumpPath
  */
 static int systemL_jumpPath( lua_State *L )
 {
    LuaJump lj;
    StarSystem *sys, *sysp;
    StarSystem **s;
-   int i, sid, jumps, pushed, h;
+   int sid, pushed, h;
    const char *start, *goal;
 
    h   = lua_toboolean(L,3);
 
    /* Foo to Bar */
-   if (lua_gettop(L) > 1) {
-      sys   = luaL_validsystem(L,1);
-      start = sys->name;
-      sid   = sys->id;
+   sys   = luaL_validsystem(L,1);
+   start = sys->name;
+   sid   = sys->id;
+   sysp  = luaL_validsystem(L,2);
+   goal  = sysp->name;
 
-      if (lua_isstring(L,2))
-         goal = lua_tostring(L,2);
-      else if (lua_issystem(L,2)) {
-         sysp = luaL_validsystem(L,2);
-         goal = sysp->name;
-      }
-      else NLUA_INVALID_PARAMETER(L);
-   }
-   /* Current to Foo */
-   else {
-      start = cur_system->name;
-      sid   = cur_system->id;
-      sys   = luaL_validsystem(L,1);
-      goal  = sys->name;
-   }
-
-   s = map_getJumpPath( &jumps, start, goal, 1, h, NULL );
+   s = map_getJumpPath( start, NULL, goal, 1, h, NULL, NULL );
    if (s == NULL)
       return 0;
 
@@ -513,24 +544,21 @@ static int systemL_jumpPath( lua_State *L )
    lj.srcid  = sid;
    lj.destid = s[0]->id;
 
-   lua_pushnumber(L, ++pushed); /* key. */
    lua_pushjump(L, lj);         /* value. */
-   lua_rawset(L, -3);
+   lua_rawseti(L, -2, ++pushed);
 
-   for (i=0; i<(jumps - 1); i++) {
+   for (int i=0; i<array_size(s)-1; i++) {
       lj.srcid  = s[i]->id;
       lj.destid = s[i+1]->id;
 
-      lua_pushnumber(L, ++pushed); /* key. */
       lua_pushjump(L, lj);         /* value. */
-      lua_rawset(L, -3);
+      lua_rawseti(L, -2, ++pushed);
    }
-   free(s);
+   array_free(s);
 
    return 1;
 }
 
-
 /**
  * @brief Gets all the adjacent systems to a system.
  *
@@ -539,12 +567,11 @@ static int systemL_jumpPath( lua_State *L )
  *    @luatparam System s System to get adjacent systems of.
  *    @luatparam[opt=false] boolean hidden Whether or not to show hidden jumps also.
  *    @luatreturn {System,...} An ordered table with all the adjacent systems.
- * @luafunc adjacentSystems( s, hidden )
+ * @luafunc adjacentSystems
  */
 static int systemL_adjacent( lua_State *L )
 {
-   int i, id, h;
-   LuaSystem sysp;
+   int id, h;
    StarSystem *s;
 
    id = 1;
@@ -553,23 +580,20 @@ static int systemL_adjacent( lua_State *L )
 
    /* Push all adjacent systems. */
    lua_newtable(L);
-   for (i=0; i<s->njumps; i++) {
+   for (int i=0; i<array_size(s->jumps); i++) {
+      LuaSystem sysp;
       if (jp_isFlag(&s->jumps[i], JP_EXITONLY ))
          continue;
       if (!h && jp_isFlag(&s->jumps[i], JP_HIDDEN))
          continue;
       sysp = system_index( s->jumps[i].target );
-      lua_pushnumber(L, id);   /* key. */
       lua_pushsystem(L, sysp); /* value. */
-      lua_rawset(L,-3);
-
-      id++;
+      lua_rawseti(L,-2,id++);
    }
 
    return 1;
 }
 
-
 /**
  * @brief Gets all the jumps in a system.
  *
@@ -578,151 +602,65 @@ static int systemL_adjacent( lua_State *L )
  *    @luatparam System s System to get the jumps of.
  *    @luatparam[opt=false] boolean exitonly Whether to exclude exit-only jumps.
  *    @luatreturn {Jump,...} An ordered table with all the jumps.
- * @luafunc jumps( s, exitonly )
+ * @luafunc jumps
  */
 static int systemL_jumps( lua_State *L )
 {
-   int i, exitonly, pushed;
-   LuaJump lj;
-   StarSystem *s;
-
-   s = luaL_validsystem(L,1);
-   exitonly = lua_toboolean(L,2);
-   pushed = 0;
+   StarSystem *s = luaL_validsystem(L,1);
+   int exitonly = lua_toboolean(L,2);
+   int pushed = 0;
 
    /* Push all jumps. */
    lua_newtable(L);
-   for (i=0; i<s->njumps; i++) {
+   for (int i=0; i<array_size(s->jumps); i++) {
+      LuaJump lj;
       /* Skip exit-only jumps if requested. */
       if ((exitonly) && (jp_isFlag( &s->jumps[i],  JP_EXITONLY)))
-            continue;
+         continue;
 
       lj.srcid  = s->id;
       lj.destid = s->jumps[i].targetid;
-      lua_pushnumber(L,++pushed); /* key. */
       lua_pushjump(L,lj); /* value. */
-      lua_rawset(L,-3);
+      lua_rawseti(L,-2, ++pushed);
    }
 
    return 1;
 }
 
-
 /**
- * @brief Gets a random asteroid in the current system
+ * @brief Gets all the asteroid fields in a system.
  *
- * @usage anchor, ast = system.asteroid()
+ * @usage for i, s in ipairs( sys:asteroidFields() ) do -- Iterate over asteroid fields
  *
- *    @luatreturn int anchor Id of an asteroid anchor.
- *    @luatreturn int asteroid Id of an asteroid of this anchor.
- * @luafunc asteroid()
+ *    @luatparam System s System to get the asteroid fields of.
+ *    @luatreturn {Table,...} An ordered table with all the asteroid fields.
+ * @luafunc asteroidFields
  */
-static int systemL_asteroid( lua_State *L )
+static int systemL_asteroidFields( lua_State *L )
 {
-   int field, ast;
-   int bad_asteroid;
-   int i;
+   StarSystem *s = luaL_validsystem(L,1);
+   /* Push all jumps. */
+   lua_newtable(L);
+   for (int i=0; i<array_size(s->asteroids); i++) {
+      lua_newtable(L);
 
-   if (cur_system->nasteroids > 0) {
-      field = RNG(0,cur_system->nasteroids-1);
-      ast   = RNG(0,cur_system->asteroids[field].nb-1);
-      bad_asteroid = 0;
-      if ( (cur_system->asteroids[field].asteroids[ast].appearing == ASTEROID_INVISIBLE) ||
-            (cur_system->asteroids[field].asteroids[ast].appearing == ASTEROID_INIT)) {
-         /* Switch to next index until we find a valid one, or until we come full-circle. */
-         bad_asteroid = 1;
-         for (i=0; i<cur_system->asteroids[field].nb; i++) {
-            ast = (ast + 1) % cur_system->asteroids[field].nb;
-            if ( (cur_system->asteroids[field].asteroids[ast].appearing != ASTEROID_INVISIBLE) &&
-                  (cur_system->asteroids[field].asteroids[ast].appearing != ASTEROID_INIT)) {
-               bad_asteroid = 0;
-               break;
-            }
-         }
-      }
+      lua_pushinteger(L,i+1);
+      lua_setfield(L,-2,"id");
 
-      if (bad_asteroid) {
-         WARN("Failed to get a valid asteroid in field %d.", field);
-         return 0;
-      }
+      lua_pushvector(L,s->asteroids[i].pos);
+      lua_setfield(L,-2,"pos");
 
-      lua_pushnumber(L,field);
-      lua_pushnumber(L,ast);
-      return 2;
+      lua_pushnumber(L,s->asteroids[i].density);
+      lua_setfield(L,-2,"density");
+
+      lua_pushnumber(L,s->asteroids[i].radius);
+      lua_setfield(L,-2,"radius");
+
+      lua_rawseti(L,-2,i+1);
    }
-
-   return 0;
-}
-
-
-/**
- * @brief Gets the position and velocity of an asteroid
- *
- * @usage pos = system.asteroidPos( anchor, ast )
- *
- *    @luatparam int anchor Id of the asteroid anchor.
- *    @luatparam int asteroid Id of the asteroid of this anchor.
- *    @luatreturn Vec2 pos position of the asteroid.
- *    @luatreturn Vec2 vel velocity of the asteroid.
- * @luafunc asteroidPos()
- */
-static int systemL_asteroidPos( lua_State *L )
-{
-   int field, ast;
-
-   field = luaL_checkint(L,1);
-   ast   = luaL_checkint(L,2);
-
-   if ( field >= cur_system->nasteroids ) {
-      WARN(_("field index %d too high"), field);
-      return 0;
-   }
-
-   if ( ast >= cur_system->asteroids[field].nb ) {
-      WARN(_("asteroid index too high"));
-      return 0;
-   }
-
-   lua_pushvector(L, cur_system->asteroids[field].asteroids[ast].pos);
-   lua_pushvector(L, cur_system->asteroids[field].asteroids[ast].vel);
-   return 2;
-}
-
-
-/**
- * @brief Sees if a given asteroid has been destroyed recently
- *
- * @usage i = system.asteroidDestroyed( anchor, ast )
- *
- *    @luatparam int anchor Id of the asteroid anchor.
- *    @luatparam int asteroid Id of the asteroid of this anchor.
- *    @luatreturn bool i true if the asteroid was destroyed.
- * @luafunc asteroidDestroyed()
- */
-static int systemL_asteroidDestroyed( lua_State *L )
-{
-   int field, ast, isdestroyed;
-
-   field = luaL_checkint(L,1);
-   ast   = luaL_checkint(L,2);
-
-   if ( field >= cur_system->nasteroids ) {
-      WARN(_("field index %d too high"), field);
-      return 0;
-   }
-
-   if ( ast >= cur_system->asteroids[field].nb ) {
-      WARN(_("asteroid index too high"));
-      return 0;
-   }
-
-   /* If the asteroid is re-appearing, it was destroyed recently. */
-   isdestroyed = (cur_system->asteroids[field].asteroids[ast].appearing == ASTEROID_GROWING);
-   lua_pushboolean(L, isdestroyed);
    return 1;
 }
 
-
 /**
  * @brief Adds a gatherable object
  *
@@ -730,63 +668,52 @@ static int systemL_asteroidDestroyed( lua_State *L )
  *
  *    @luatparam string commodity name of the commodity.
  *    @luatparam int nb quantity of commodity in the gatherable .
- *    @luatparam Vec2 pos position of the gatherable.
- *    @luatparam Vec2 vel velocity of the gatherable.
- *    @luatparam[opt] number lifelength Lifelength of the gatherable.
+ *    @luatparam[opt=vec2.new(0,0)] Vec2 pos position of the gatherable.
+ *    @luatparam[opt=vec2.new(0,0)] Vec2 vel velocity of the gatherable.
+ *    @luatparam[opt] number lifelength Lifelength of the gatherable in seconds.
+ *    @luatparam[opt=false] boolean If true, the gatherable can only be gathered by player.
  *    @luatreturn int i Id of the created gatherable object.
- * @luafunc addGatherable( commodity, nb, pos, vel, lifelength )
+ * @luafunc addGatherable
  */
 static int systemL_addGatherable( lua_State *L )
 {
    int nb;
-   const char *name;
+   unsigned int player_only;
    Commodity *commodity;
-   Vector2d *pos, *vel;
+   vec2 *pos, *vel;
+   vec2 zero = { .x = 0., .y = 0., .mod = 0., .angle = 0. };
    double lifelength;
 
    /* Handle parameters. */
-   name = luaL_checkstring(L,1);
+   commodity = luaL_validcommodity( L, 1 );
    nb = luaL_checkint(L,2);
-   pos = luaL_checkvector(L,3);
-   vel = luaL_checkvector(L,4);
-   if (lua_gettop(L) > 4)
-      lifelength = luaL_checknumber(L,5);
-   else
-      lifelength = -1.; /* This means random life length. */
+   pos = luaL_optvector(L,3,&zero);
+   vel = luaL_optvector(L,4,&zero);
+   lifelength = luaL_optnumber(L,5, -1.); /* -1. means random life length. */
+   player_only = lua_toboolean(L,6);
 
-   /* Get commodity. */
-   commodity = commodity_get( name );
-   if (commodity == NULL) {
-      NLUA_ERROR(L,_("Commodity '%s' not found!"), name);
-      return 0;
-   }
-
-   return gatherable_init( commodity, *pos, *vel, lifelength, nb );
+   lua_pushnumber( L, gatherable_init( commodity, pos, vel, lifelength, nb, player_only ) );
+   return 1;
 }
 
-
 /**
  * @brief Returns the factions that have presence in a system and their respective presence values.
  *        Faction names are internal -- can be passed to other functions as a faction identifier, but
  *        should not be shown to the user without being translated by _().
  *
  *  @usage if sys:presences()["Empire"] then -- Checks to see if Empire has ships in the system
- *  @usage if sys:presences()[faction.get("Pirate")] then -- Checks to see if the Pirates have ships in the system
+ *  @usage if sys:presences()[faction.get("Dvaered"):nameRaw()] then -- Checks to see if the Dvaered have ships in the system
  *
  *    @luatparam System s System to get the factional presences of.
  *    @luatreturn {Faction,...} A table with the factions that have presence in the system.
- * @luafunc presences( s )
+ * @luafunc presences
  */
 static int systemL_presences( lua_State *L )
 {
-   StarSystem *s;
-   int i;
-
-   s = luaL_validsystem(L,1);
-
+   StarSystem *s = luaL_validsystem(L,1);
    /* Return result in table */
    lua_newtable(L);
-   for (i=0; i<s->npresence; i++) {
+   for (int i=0; i<array_size(s->presence); i++) {
       /* Only return positive presences. */
       if (s->presence[i].value <= 0)
          continue;
@@ -799,40 +726,28 @@ static int systemL_presences( lua_State *L )
    return 1;
 }
 
-
 /**
- * @brief Gets the planets in a system.
+ * @brief Gets the spobs in a system.
  *
- * @usage for key, planet in ipairs( sys:planets() ) do -- Iterate over planets in system
- * @usage if \#sys:planets() > 0 then -- System has planets
+ * @usage for key, spob in ipairs( sys:spobs() ) do -- Iterate over spobs in system
+ * @usage if \#sys:spobs() > 0 then -- System has spobs
  *
- *    @luatparam System s System to get planets of
- *    @luatreturn {Planet,...} A table with all the planets
- * @luafunc planets( s )
+ *    @luatparam System s System to get spobs of
+ *    @luatreturn {Spob,...} A table with all the spobs
+ * @luafunc spobs
  */
-static int systemL_planets( lua_State *L )
+static int systemL_spobs( lua_State *L )
 {
-   int i, key;
-   StarSystem *s;
-
-   s = luaL_validsystem(L,1);
-
-   /* Push all planets. */
+   const StarSystem *s = luaL_validsystem(L,1);
+   /* Push all spobs. */
    lua_newtable(L);
-   key = 0;
-   for (i=0; i<s->nplanets; i++) {
-      if (s->planets[i]->real == ASSET_REAL) {
-         key++;
-         lua_pushnumber(L,key); /* key */
-         lua_pushplanet(L,planet_index( s->planets[i] )); /* value */
-         lua_rawset(L,-3);
-      }
+   for (int i=0; i<array_size(s->spobs); i++) {
+      lua_pushspob(L,spob_index( s->spobs[i] )); /* value */
+      lua_rawseti(L,-2,i+1);
    }
-
    return 1;
 }
 
-
 /**
  * @brief Gets the presence in the system.
  *
@@ -848,69 +763,65 @@ static int systemL_planets( lua_State *L )
  *
  *    @luatparam System s System to get presence level of.
  *    @luatreturn number The presence level in sys (absolute value).
- * @luafunc presence( s )
+ * @luafunc presence
  */
 static int systemL_presence( lua_State *L )
 {
    StarSystem *sys;
    int *fct;
-   int nfct;
-   double presence, v;
-   int i, f, used;
-   const char *cmd;
+   double presence;
+   int used;
 
    /* Get parameters. */
    sys = luaL_validsystem(L, 1);
 
    /* Allow fall-through. */
    used = 0;
+   fct  = NULL;
 
    /* Get the second parameter. */
    if (lua_isstring(L, 2)) {
       /* A string command has been given. */
-      cmd  = lua_tostring(L, 2);
-      nfct = 0;
+      const char *cmd = lua_tostring(L, 2);
       used = 1;
 
       /* Check the command string and get the appropriate faction group.*/
       if (strcmp(cmd, "all") == 0)
-         fct = faction_getGroup(&nfct, 0);
+         fct = faction_getGroup(0);
       else if (strcmp(cmd, "friendly") == 0)
-         fct = faction_getGroup(&nfct, 1);
+         fct = faction_getGroup(1);
       else if (strcmp(cmd, "hostile") == 0)
-         fct = faction_getGroup(&nfct, 3);
+         fct = faction_getGroup(3);
       else if (strcmp(cmd, "neutral") == 0)
-         fct = faction_getGroup(&nfct, 2);
+         fct = faction_getGroup(2);
       else /* Invalid command string. */
          used = 0;
    }
 
    if (!used) {
       /* A faction id was given. */
-      f      = luaL_validfaction(L, 2);
-      nfct   = 1;
-      fct    = malloc(sizeof(int));
-      fct[0] = f;
+      int f  = luaL_validfaction(L, 2);
+      fct    = array_create(int);
+      array_push_back(&fct, f);
    }
 
    /* Add up the presence values. */
    presence = 0;
-   for (i=0; i<nfct; i++) {
+   for (int i=0; i<array_size(fct); i++) {
       /* Only count positive presences. */
-      v = system_getPresence( sys, fct[i] );
+      double v = system_getPresence( sys, fct[i] );
       if (v > 0)
          presence += v;
    }
 
    /* Clean up after ourselves. */
-   free(fct);
+   array_free(fct);
 
    /* Push it back to Lua. */
    lua_pushnumber(L, presence);
    return 1;
 }
 
-
 /**
  * @brief Gets the radius of the system.
  *
@@ -920,20 +831,15 @@ static int systemL_presence( lua_State *L )
  *
  *    @luatparam System s System to get the radius of.
  *    @luatreturn number The radius of the system.
- * @luafunc radius( s )
+ * @luafunc radius
  */
 static int systemL_radius( lua_State *L )
 {
-   StarSystem *sys;
-
-   /* Get parameters. */
-   sys = luaL_validsystem(L, 1);
-
+   StarSystem *sys = luaL_validsystem(L, 1);
    lua_pushnumber( L, sys->radius );
    return 1;
 }
 
-
 /**
  * @brief Checks to see if a system is known by the player.
  *
@@ -941,33 +847,30 @@ static int systemL_radius( lua_State *L )
  *
  *    @luatparam System s System to check if the player knows.
  *    @luatreturn boolean true if the player knows the system.
- * @luafunc known( s )
+ * @luafunc known
  */
 static int systemL_isknown( lua_State *L )
 {
-   StarSystem *sys = luaL_validsystem(L, 1);
+   const StarSystem *sys = luaL_validsystem(L, 1);
    lua_pushboolean(L, sys_isKnown(sys));
    return 1;
 }
 
-
 /**
  * @brief Sets a system's known state.
  *
  * @usage s:setKnown( false ) -- Makes system unknown.
  *    @luatparam System  s System to set known.
  *    @luatparam[opt=false] boolean b Whether or not to set as known.
- *    @luatparam[opt=false] boolean r Whether or not to iterate over the system's assets and jump points.
- * @luafunc setKnown( s, b, r )
+ *    @luatparam[opt=false] boolean r Whether or not to iterate over the system's spobs and jump points.
+ * @luafunc setKnown
  */
 static int systemL_setknown( lua_State *L )
 {
-   int b, r, i;
+   int b, r;
    StarSystem *sys;
 
-   NLUA_CHECKRW(L);
-
-   r = 0;
+   r   = 0;
    sys = luaL_validsystem(L, 1);
    b   = lua_toboolean(L, 2);
    if (lua_gettop(L) > 2)
@@ -980,87 +883,142 @@ static int systemL_setknown( lua_State *L )
 
    if (r) {
       if (b) {
-         for (i=0; i < sys->nplanets; i++)
-            planet_setKnown( sys->planets[i] );
-         for (i=0; i < sys->njumps; i++)
+         for (int i=0; i < array_size(sys->spobs); i++)
+            spob_setKnown( sys->spobs[i] );
+         for (int i=0; i < array_size(sys->jumps); i++)
             jp_setFlag( &sys->jumps[i], JP_KNOWN );
-     }
-     else {
-         for (i=0; i < sys->nplanets; i++)
-            planet_rmFlag( sys->planets[i], PLANET_KNOWN );
-         for (i=0; i < sys->njumps; i++)
+      }
+      else {
+         for (int i=0; i < array_size(sys->spobs); i++)
+            spob_rmFlag( sys->spobs[i], SPOB_KNOWN );
+         for (int i=0; i < array_size(sys->jumps); i++)
             jp_rmFlag( &sys->jumps[i], JP_KNOWN );
-     }
+      }
    }
 
    /* Update outfits image array. */
    outfits_updateEquipmentOutfits();
+   ovr_refresh(); /* Update overlay as necessary. */
 
    return 0;
 }
 
+/**
+ * @brief Checks to see if a system is hidden by the player.
+ *
+ * @usage b = s:hidden()
+ *
+ *    @luatparam System s System to check if the player knows.
+ *    @luatreturn boolean true if the player knows the system.
+ * @luafunc hidden
+ */
+static int systemL_hidden( lua_State *L )
+{
+   const StarSystem *sys = luaL_validsystem(L, 1);
+   lua_pushboolean(L, sys_isFlag( sys, SYSTEM_HIDDEN ));
+   return 1;
+}
+
+/**
+ * @brief Sets a system to be hidden to the player.
+ *
+ * @usage s:setHidden( true )
+ *
+ *    @luatparam System s System to check if the player knows.
+ *    @luatparam boolean hide Whether or not to hide the system.
+ * @luafunc hidden
+ */
+static int systemL_setHidden( lua_State *L )
+{
+   StarSystem *sys = luaL_validsystem(L, 1);
+   int b = lua_toboolean(L,2);
+   if (b)
+      sys_setFlag( sys, SYSTEM_HIDDEN );
+   else
+      sys_rmFlag( sys, SYSTEM_HIDDEN );
+   return 0;
+}
 
 /**
  * @brief Clears the system markers.
  *
  * This can be dangerous and clash with other missions, do not try this at home kids.
  *
- * @usage system.mrkClear()
+ * @usage system.markerClear()
  *
- * @luafunc mrkClear()
+ * @luafunc markerClear
  */
-static int systemL_mrkClear( lua_State *L )
+static int systemL_markerClear( lua_State *L )
 {
    (void) L;
-   NLUA_CHECKRW(L);
    ovr_mrkClear();
    return 0;
 }
 
-
 /**
  * @brief Adds a system marker.
  *
- * @usage mrk_id = system.mrkAdd( "Hello", vec2.new( 50, 30 ) ) -- Creates a marker at (50,30)
+ * @usage mrk_id = system.markerAdd( vec2.new( 50, 30 ), "Hello" ) -- Creates a marker at (50,30)
  *
- *    @luatparam string str String to display next to marker.
  *    @luatparam Vec2 v Position to display marker at.
+ *    @luatparam[opt] string str String to display next to marker.
+ *    @luatparam[opt] number If specified, changes the marker to circle type marker and specifies the radius of the circle to use.
  *    @luatreturn number The id of the marker.
- * @luafunc mrkAdd( str, v )
+ * @luafunc markerAdd
  */
-static int systemL_mrkAdd( lua_State *L )
+static int systemL_markerAdd( lua_State *L )
 {
    const char *str;
-   Vector2d *vec;
+   vec2 *vec;
    unsigned int id;
-
-   NLUA_CHECKRW(L);
+   double r;
 
    /* Handle parameters. */
-   str   = luaL_checkstring( L, 1 );
-   vec   = luaL_checkvector( L, 2 );
+   vec = luaL_checkvector( L, 1 );
+   str = luaL_optstring( L, 2, NULL );
+   r = luaL_optnumber( L, 3, -1. );
 
    /* Create marker. */
-   id    = ovr_mrkAddPoint( str, vec->x, vec->y );
+   if (r < 0.)
+      id = ovr_mrkAddPoint( str, vec->x, vec->y );
+   else
+      id = ovr_mrkAddCircle( str, vec->x, vec->y, r );
    lua_pushnumber( L, id );
    return 1;
 }
 
-
 /**
  * @brief Removes a system marker.
  *
- * @usage system.mrkRm( mrk_id ) -- Removes a marker by mrk_id
+ * @usage system.markerRm( mrk_id ) -- Removes a marker by mrk_id
  *
  *    @luatparam number id ID of the marker to remove.
- * @luafunc mrkRm( id )
+ * @luafunc markerRm
  */
-static int systemL_mrkRm( lua_State *L )
+static int systemL_markerRm( lua_State *L )
 {
-   unsigned int id;
-   NLUA_CHECKRW(L);
-   id = luaL_checklong( L, 1 );
+   unsigned int id = luaL_checklong( L, 1 );
    ovr_mrkRm( id );
    return 0;
 }
 
+/**
+ * @brief Gets the system tags.
+ *
+ * @usage if system.cur():tags["fancy"] then -- Has "fancy" tag
+ *
+ *    @luatparam System s System to get tags of.
+ *    @luatreturn table Table of tags where the name is the key and true is the value.
+ * @luafunc tags
+ */
+static int systemL_tags( lua_State *L )
+{
+   StarSystem *s = luaL_validsystem(L,1);
+   lua_newtable(L);
+   for (int i=0; i<array_size(s->tags); i++) {
+      lua_pushstring(L,s->tags[i]);
+      lua_pushboolean(L,1);
+      lua_rawset(L,-3);
+   }
+   return 1;
+}
