@@ -1,28 +1,30 @@
 /*
  * See Licensing and Copyright notice in naev.h
  */
-
 /**
  * @file nlua_vec2.c
  *
  * @brief Handles the Lua vector handling bindings.
  *
- * These bindings control the planets and systems.
+ * These bindings control the spobs and systems.
  */
+/** @cond */
+#include <lauxlib.h>
+
+#include "naev.h"
+/** @endcond */
 
 #include "nlua_vec2.h"
 
-#include "naev.h"
-
-#include <lauxlib.h>
-
-#include "nluadef.h"
 #include "log.h"
-
+#include "nluadef.h"
+#include "collision.h"
 
 /* Vector metatable methods */
 static int vectorL_new( lua_State *L );
 static int vectorL_newP( lua_State *L );
+static int vectorL_copy( lua_State *L );
+static int vectorL_tostring( lua_State *L );
 static int vectorL_add__( lua_State *L );
 static int vectorL_add( lua_State *L );
 static int vectorL_sub__( lua_State *L );
@@ -31,6 +33,9 @@ static int vectorL_mul__( lua_State *L );
 static int vectorL_mul( lua_State *L );
 static int vectorL_div__( lua_State *L );
 static int vectorL_div( lua_State *L );
+static int vectorL_unm( lua_State *L );
+static int vectorL_dot( lua_State *L );
+static int vectorL_cross( lua_State *L );
 static int vectorL_get( lua_State *L );
 static int vectorL_polar( lua_State *L );
 static int vectorL_set( lua_State *L );
@@ -38,9 +43,15 @@ static int vectorL_setP( lua_State *L );
 static int vectorL_distance( lua_State *L );
 static int vectorL_distance2( lua_State *L );
 static int vectorL_mod( lua_State *L );
+static int vectorL_angle( lua_State *L );
+static int vectorL_normalize( lua_State *L );
+static int vectorL_collideLineLine( lua_State *L );
+static int vectorL_collideCircleLine( lua_State *L );
 static const luaL_Reg vector_methods[] = {
    { "new", vectorL_new },
    { "newP", vectorL_newP },
+   { "copy", vectorL_copy },
+   { "__tostring", vectorL_tostring },
    { "__add", vectorL_add },
    { "add", vectorL_add__ },
    { "__sub", vectorL_sub },
@@ -49,6 +60,9 @@ static const luaL_Reg vector_methods[] = {
    { "mul", vectorL_mul__ },
    { "__div", vectorL_div },
    { "div", vectorL_div__ },
+   { "__unm", vectorL_unm},
+   { "dot", vectorL_dot },
+   { "cross", vectorL_cross },
    { "get", vectorL_get },
    { "polar", vectorL_polar },
    { "set", vectorL_set },
@@ -56,10 +70,13 @@ static const luaL_Reg vector_methods[] = {
    { "dist", vectorL_distance },
    { "dist2", vectorL_distance2 },
    { "mod", vectorL_mod },
+   { "angle", vectorL_angle },
+   { "normalize", vectorL_normalize },
+   { "collideLineLine", vectorL_collideLineLine },
+   { "collideCircleLine", vectorL_collideCircleLine },
    {0,0}
 }; /**< Vector metatable methods. */
 
-
 /**
  * @brief Loads the vector metatable.
  *
@@ -72,7 +89,6 @@ int nlua_loadVector( nlua_env env )
    return 0;
 }
 
-
 /**
  * @brief Represents a 2D vector in Lua.
  *
@@ -98,23 +114,23 @@ int nlua_loadVector( nlua_env env )
  *
  *    @param L Lua state to get vector from.
  *    @param ind Index position of vector.
- *    @return The Vector2d at ind.
+ *    @return The vec2 at ind.
  */
-Vector2d* lua_tovector( lua_State *L, int ind )
+vec2* lua_tovector( lua_State *L, int ind )
 {
-   return (Vector2d*) lua_touserdata(L,ind);
+   return (vec2*) lua_touserdata(L,ind);
 }
 /**
  * @brief Gets vector at index making sure type is valid.
  *
  *    @param L Lua state to get vector from.
  *    @param ind Index position of vector.
- *    @return The Vector2d at ind.
+ *    @return The vec2 at ind.
  */
-Vector2d* luaL_checkvector( lua_State *L, int ind )
+vec2* luaL_checkvector( lua_State *L, int ind )
 {
    if (lua_isvector(L,ind))
-      return (Vector2d*) lua_touserdata(L,ind);
+      return (vec2*) lua_touserdata(L,ind);
    luaL_typerror(L, ind, VECTOR_METATABLE);
    return NULL;
 }
@@ -126,10 +142,9 @@ Vector2d* luaL_checkvector( lua_State *L, int ind )
  *    @param vec Vector to push.
  *    @return Vector just pushed.
  */
-Vector2d* lua_pushvector( lua_State *L, Vector2d vec )
+vec2* lua_pushvector( lua_State *L, vec2 vec )
 {
-   Vector2d *v;
-   v = (Vector2d*) lua_newuserdata(L, sizeof(Vector2d));
+   vec2 *v = (vec2*) lua_newuserdata(L, sizeof(vec2));
    *v = vec;
    luaL_getmetatable(L, VECTOR_METATABLE);
    lua_setmetatable(L, -2);
@@ -165,26 +180,27 @@ int lua_isvector( lua_State *L, int ind )
  * @usage vec2.new( 5, 3 ) -- creates a vector at (5,3)
  * @usage vec2.new() -- creates a vector at (0,0)
  *
- *    @luatparam number x If set, the X value for the new vector.
- *    @luatparam number y If set, the Y value for the new vector.
+ *    @luatparam[opt=0] number x If set, the X value for the new vector.
+ *    @luatparam[opt=x] number y If set, the Y value for the new vector.
  *    @luatreturn Vec2 The new vector.
- * @luafunc new( x, y )
+ * @luafunc new
  */
 static int vectorL_new( lua_State *L )
 {
-   Vector2d v;
+   vec2 v;
    double x, y;
 
-   if ((lua_gettop(L) > 1) && lua_isnumber(L,1) && lua_isnumber(L,2)) {
-      x = lua_tonumber(L,1);
-      y = lua_tonumber(L,2);
-   }
-   else {
+   if (!lua_isnoneornil(L,1))
+      x = luaL_checknumber(L,1);
+   else
       x = 0.;
-      y = 0.;
-   }
 
-   vect_cset( &v, x, y );
+   if (!lua_isnoneornil(L,2))
+      y = luaL_checknumber(L,2);
+   else
+      y = x;
+
+   vec2_cset( &v, x, y );
    lua_pushvector(L, v);
    return 1;
 }
@@ -196,29 +212,60 @@ static int vectorL_new( lua_State *L )
  * @usage vec2.newP() -- creates a vector at (0,0)
  *
  *    @luatparam[opt=0] number m If set, the modulus for the new vector.
- *    @luatparam[opt=0] number a If set, the angle for the new vector, in degrees.
+ *    @luatparam[opt=0] number a If set, the angle for the new vector, in radians.
  *    @luatreturn Vec2 The new vector.
- * @luafunc newP( m, a )
+ * @luafunc newP
  */
 static int vectorL_newP( lua_State *L )
 {
-   Vector2d v;
+   vec2 v;
    double m, a;
 
-   if ((lua_gettop(L) > 1) && lua_isnumber(L,1) && lua_isnumber(L,2)) {
-      m = lua_tonumber(L, 1);
-      a = lua_tonumber(L, 2) / 180. * M_PI;
-   }
-   else {
+   if (!lua_isnoneornil(L,1))
+      m = luaL_checknumber(L, 1);
+   else
       m = 0.;
-      a = 0.;
-   }
 
-   vect_pset( &v, m, a );
+   if (!lua_isnoneornil(L,2))
+      a = luaL_checknumber(L, 2);
+   else
+      a = 0.;
+
+   vec2_pset( &v, m, a );
    lua_pushvector(L, v);
    return 1;
 }
 
+/**
+ * @brief Copies a vector.
+ *
+ *    @luatparam Vec2 v Vector to copy.
+ *    @luatreturn Vec2 A copy of v.
+ * @luafunc copy
+ */
+static int vectorL_copy( lua_State *L )
+{
+   const vec2 *v = luaL_checkvector(L,1);
+   lua_pushvector( L, *v );
+   return 1;
+}
+
+/**
+ * @brief Converts a vector to a string.
+ *
+ *    @luatparam Vector v Vector to convert to as string.
+ *    @luatreturn string String version of v.
+ * @luafunc __tostring
+ */
+static int vectorL_tostring( lua_State *L )
+{
+   char buf[STRMAX_SHORT];
+   const vec2 *v = luaL_checkvector(L,1);
+   snprintf( buf, sizeof(buf), "vec2( %g, %g )", v->x, v->y );
+   lua_pushstring(L, buf);
+   return 1;
+}
+
 /**
  * @brief Adds two vectors or a vector and some cartesian coordinates.
  *
@@ -229,68 +276,70 @@ static int vectorL_newP( lua_State *L )
  * @usage my_vec:add( your_vec )
  * @usage my_vec:add( 5, 3 )
  *
- *    @luatparam Vector v Vector getting stuff subtracted from.
+ *    @luatparam Vector v Vector getting stuff added to.
  *    @luatparam number|Vec2 x X coordinate or vector to add to.
  *    @luatparam number|nil y Y coordinate or nil to add to.
  *    @luatreturn Vec2 The result of the vector operation.
- * @luafunc add( v, x, y )
+ * @luafunc add
  */
 static int vectorL_add( lua_State *L )
 {
-   Vector2d vout, *v1, *v2;
+   vec2 vout, *v1;
    double x, y;
 
-   /* Get self. */
-   v1    = luaL_checkvector(L,1);
-
-   /* Get rest of parameters. */
-   v2 = NULL;
-   if (lua_isvector(L,2)) {
-      v2 = lua_tovector(L,2);
-      x = v2->x;
-      y = v2->y;
-   }
-   else if ((lua_gettop(L) > 2) && lua_isnumber(L,2) && lua_isnumber(L,3)) {
-      x = lua_tonumber(L,2);
-      y = lua_tonumber(L,3);
+   if (lua_isnumber(L,1)) {
+      x = y = lua_tonumber(L,1);
+      v1 = luaL_checkvector(L,2);
    }
    else {
-      NLUA_INVALID_PARAMETER(L);
-      return 0;
+      /* Get self. */
+      v1    = luaL_checkvector(L,1);
+
+      /* Get rest of parameters. */
+      if (lua_isvector(L,2)) {
+         const vec2 *v2 = lua_tovector(L,2);
+         x = v2->x;
+         y = v2->y;
+      }
+      else {
+         x = luaL_checknumber(L,2);
+         if (!lua_isnoneornil(L,3))
+            y = luaL_checknumber(L,3);
+         else
+            y = x;
+      }
    }
 
    /* Actually add it */
-   vect_cset( &vout, v1->x + x, v1->y + y );
+   vec2_cset( &vout, v1->x + x, v1->y + y );
    lua_pushvector( L, vout );
 
    return 1;
 }
 static int vectorL_add__( lua_State *L )
 {
-   Vector2d *v1, *v2;
+   vec2 *v1;
    double x, y;
 
    /* Get self. */
-   v1    = luaL_checkvector(L,1);
+   v1 = luaL_checkvector(L,1);
 
    /* Get rest of parameters. */
-   v2 = NULL;
    if (lua_isvector(L,2)) {
-      v2 = lua_tovector(L,2);
+      const vec2 *v2 = lua_tovector(L,2);
       x = v2->x;
       y = v2->y;
    }
-   else if ((lua_gettop(L) > 2) && lua_isnumber(L,2) && lua_isnumber(L,3)) {
-      x = lua_tonumber(L,2);
-      y = lua_tonumber(L,3);
-   }
    else {
-      NLUA_INVALID_PARAMETER(L);
-      return 0;
+      x = luaL_checknumber(L,2);
+      if (!lua_isnoneornil(L,3))
+         y = luaL_checknumber(L,3);
+      else
+         y = x;
    }
 
    /* Actually add it */
-   vect_cset( v1, v1->x + x, v1->y + y );
+   vec2_cset( v1, v1->x + x, v1->y + y );
    lua_pushvector( L, *v1 );
 
    return 1;
@@ -310,63 +359,60 @@ static int vectorL_add__( lua_State *L )
  *    @luatparam number|Vec2 x X coordinate or vector to subtract.
  *    @luatparam number|nil y Y coordinate or nil to subtract.
  *    @luatreturn Vec2 The result of the vector operation.
- * @luafunc sub( v, x, y )
+ * @luafunc sub
  */
 static int vectorL_sub( lua_State *L )
 {
-   Vector2d vout, *v1, *v2;
+   const vec2 *v1;
+   vec2 vout;
    double x, y;
 
    /* Get self. */
    v1    = luaL_checkvector(L,1);
 
    /* Get rest of parameters. */
-   v2 = NULL;
    if (lua_isvector(L,2)) {
-      v2 = lua_tovector(L,2);
+      const vec2 *v2 = lua_tovector(L,2);
       x = v2->x;
       y = v2->y;
    }
-   else if ((lua_gettop(L) > 2) && lua_isnumber(L,2) && lua_isnumber(L,3)) {
-      x = lua_tonumber(L,2);
-      y = lua_tonumber(L,3);
-   }
    else {
-      NLUA_INVALID_PARAMETER(L);
-      return 0;
+      x = luaL_checknumber(L,2);
+      if (!lua_isnoneornil(L,3))
+         y = luaL_checknumber(L,3);
+      else
+         y = x;
    }
 
    /* Actually add it */
-   vect_cset( &vout, v1->x - x, v1->y - y );
+   vec2_cset( &vout, v1->x - x, v1->y - y );
    lua_pushvector( L, vout );
    return 1;
 }
 static int vectorL_sub__( lua_State *L )
 {
-   Vector2d *v1, *v2;
+   vec2 *v1;
    double x, y;
 
    /* Get self. */
    v1    = luaL_checkvector(L,1);
 
    /* Get rest of parameters. */
-   v2 = NULL;
    if (lua_isvector(L,2)) {
-      v2 = lua_tovector(L,2);
+      const vec2 *v2 = lua_tovector(L,2);
       x = v2->x;
       y = v2->y;
    }
-   else if ((lua_gettop(L) > 2) && lua_isnumber(L,2) && lua_isnumber(L,3)) {
-      x = lua_tonumber(L,2);
-      y = lua_tonumber(L,3);
-   }
    else {
-      NLUA_INVALID_PARAMETER(L);
-      return 0;
+      x = luaL_checknumber(L,2);
+      if (!lua_isnoneornil(L,3))
+         y = luaL_checknumber(L,3);
+      else
+         y = x;
    }
 
    /* Actually add it */
-   vect_cset( v1, v1->x - x, v1->y - y );
+   vec2_cset( v1, v1->x - x, v1->y - y );
    lua_pushvector( L, *v1 );
    return 1;
 }
@@ -380,33 +426,47 @@ static int vectorL_sub__( lua_State *L )
  *    @luatparam Vec2 v Vector to multiply.
  *    @luatparam number mod Amount to multiply by.
  *    @luatreturn Vec2 The result of the vector operation.
- * @luafunc mul( v, mod )
+ * @luafunc mul
  */
 static int vectorL_mul( lua_State *L )
 {
-   Vector2d vout, *v1;
-   double mod;
+   vec2 vout;
 
-   /* Get parameters. */
-   v1    = luaL_checkvector(L,1);
-   mod   = luaL_checknumber(L,2);
+   if (lua_isnumber(L,1)) {
+      double d = lua_tonumber(L,1);
+      const vec2 *v  = luaL_checkvector(L,2);
+      vec2_cset( &vout, v->x * d, v->y * d );
+   }
+   else {
+      if (lua_isnumber(L,2)) {
+         const vec2 *v  = luaL_checkvector(L,1);
+         double d = lua_tonumber(L,2);
+         vec2_cset( &vout, v->x * d, v->y * d );
+      }
+      else {
+         const vec2 *v1 = luaL_checkvector(L,1);
+         const vec2 *v2 = luaL_checkvector(L,2);
+         vec2_cset( &vout, v1->x * v2->x, v1->y * v2->y );
+      }
+   }
 
    /* Actually add it */
-   vect_cset( &vout, v1->x * mod, v1->y * mod );
    lua_pushvector( L, vout );
    return 1;
 }
 static int vectorL_mul__( lua_State *L )
 {
-   Vector2d *v1;
-   double mod;
-
-   /* Get parameters. */
-   v1    = luaL_checkvector(L,1);
-   mod   = luaL_checknumber(L,2);
+   vec2 *v1 = luaL_checkvector(L,1);
+   if (lua_isnumber(L,2)) {
+      double mod = luaL_checknumber(L,2);
+      vec2_cset( v1, v1->x * mod, v1->y * mod );
+   }
+   else {
+      const vec2 *v2 = luaL_checkvector(L,2);
+      vec2_cset( v1, v1->x * v2->x, v1->y * v2->y );
+   }
 
    /* Actually add it */
-   vect_cset( v1, v1->x * mod, v1->y * mod );
    lua_pushvector( L, *v1 );
    return 1;
 }
@@ -420,37 +480,79 @@ static int vectorL_mul__( lua_State *L )
  *    @luatparam Vec2 v Vector to divide.
  *    @luatparam number mod Amount to divide by.
  *    @luatreturn Vec2 The result of the vector operation.
- * @luafunc div( v, mod )
+ * @luafunc div
  */
 static int vectorL_div( lua_State *L )
 {
-   Vector2d vout, *v1;
-   double mod;
+   vec2 vout;
+   const vec2 *v1 = luaL_checkvector(L,1);
+   if (lua_isnumber(L,2)) {
+      double mod = lua_tonumber(L,2);
+      vec2_cset( &vout, v1->x / mod, v1->y / mod );
+   }
+   else {
+      const vec2 *v2 = luaL_checkvector(L,2);
+      vec2_cset( &vout, v1->x / v2->x, v1->y / v2->y );
+   }
 
-   /* Get parameters. */
-   v1    = luaL_checkvector(L,1);
-   mod   = luaL_checknumber(L,2);
-
-   /* Actually add it */
-   vect_cset( &vout, v1->x / mod, v1->y / mod );
    lua_pushvector( L, vout );
    return 1;
 }
 static int vectorL_div__( lua_State *L )
 {
-   Vector2d *v1;
-   double mod;
+   vec2 *v1 = luaL_checkvector(L,1);
+   if (lua_isnumber(L,2)) {
+      double mod = lua_tonumber(L,2);
+      vec2_cset( v1, v1->x / mod, v1->y / mod );
+   }
+   else {
+      const vec2 *v2 = luaL_checkvector(L,2);
+      vec2_cset( v1, v1->x / v2->x, v1->y / v2->y );
+   }
 
-   /* Get parameters. */
-   v1    = luaL_checkvector(L,1);
-   mod   = luaL_checknumber(L,2);
-
-   /* Actually add it */
-   vect_cset( v1, v1->x / mod, v1->y / mod );
    lua_pushvector( L, *v1 );
    return 1;
 }
+static int vectorL_unm( lua_State *L )
+{
+   vec2 vout;
+   const vec2 *vin = luaL_checkvector(L,1);
+   vec2_cset( &vout, -vin->x, -vin->y );
+   lua_pushvector( L, vout );
+   return 1;
+}
 
+/**
+ * @brief Dot product of two vectors.
+ *
+ *    @luatparam Vec2 a First vector.
+ *    @luatparam Vec2 b Second vector.
+ *    @luatreturn number The dot product.
+ * @luafunc dot
+ */
+static int vectorL_dot( lua_State *L )
+{
+   const vec2 *a = luaL_checkvector(L,1);
+   const vec2 *b = luaL_checkvector(L,2);
+   lua_pushnumber( L, a->x*b->x + a->y*b->y );
+   return 1;
+}
+
+/**
+ * @brief Cross product of two vectors.
+ *
+ *    @luatparam Vec2 a First vector.
+ *    @luatparam Vec2 b Second vector.
+ *    @luatreturn number The cross product.
+ * @luafunc cross
+ */
+static int vectorL_cross( lua_State *L )
+{
+   const vec2 *a = luaL_checkvector(L,1);
+   const vec2 *b = luaL_checkvector(L,2);
+   lua_pushnumber( L, a->x*b->y - a->y*b->x );
+   return 1;
+}
 
 /**
  * @brief Gets the cartesian positions of the vector.
@@ -460,15 +562,11 @@ static int vectorL_div__( lua_State *L )
  *    @luatparam Vec2 v Vector to get position of.
  *    @luatreturn number X position of the vector.
  *    @luatreturn number Y position of the vector.
- * @luafunc get(v)
+ * @luafunc get
  */
 static int vectorL_get( lua_State *L )
 {
-   Vector2d *v1;
-
-   /* Get self. */
-   v1 = luaL_checkvector(L,1);
-
+   const vec2 *v1 = luaL_checkvector(L,1);
    /* Push the vector. */
    lua_pushnumber(L, v1->x);
    lua_pushnumber(L, v1->y);
@@ -478,24 +576,20 @@ static int vectorL_get( lua_State *L )
 /**
  * @brief Gets polar coordinates of a vector.
  *
- * The angle is in degrees, not radians.
+ * The angle is in radians.
  *
  * @usage modulus, angle = my_vec:polar()
  *
  *    @luatparam Vec2 v Vector to get polar coordinates of.
  *    @luatreturn number The modulus of the vector.
  *    @luatreturn number The angle of the vector.
- * @luafunc polar(v)
+ * @luafunc polar
  */
 static int vectorL_polar( lua_State *L )
 {
-   Vector2d *v1;
-
-   /* Get self. */
-   v1 = luaL_checkvector(L,1);
-
+   const vec2 *v1 = luaL_checkvector(L,1);
    lua_pushnumber(L, VMOD(*v1));
-   lua_pushnumber(L, VANGLE(*v1)*180./M_PI);
+   lua_pushnumber(L, VANGLE(*v1));
    return 2;
 }
 
@@ -507,11 +601,11 @@ static int vectorL_polar( lua_State *L )
  *    @luatparam Vec2 v Vector to set coordinates of.
  *    @luatparam number x X coordinate to set.
  *    @luatparam number y Y coordinate to set.
- * @luafunc set( v, x, y )
+ * @luafunc set
  */
 static int vectorL_set( lua_State *L )
 {
-   Vector2d *v1;
+   vec2 *v1;
    double x, y;
 
    /* Get parameters. */
@@ -519,7 +613,7 @@ static int vectorL_set( lua_State *L )
    x  = luaL_checknumber(L,2);
    y  = luaL_checknumber(L,3);
 
-   vect_cset( v1, x, y );
+   vec2_cset( v1, x, y );
    return 0;
 }
 
@@ -530,20 +624,20 @@ static int vectorL_set( lua_State *L )
  *
  *    @luatparam Vec2 v Vector to set coordinates of.
  *    @luatparam number m Modulus to set.
- *    @luatparam number a Angle to set, in degrees.
- * @luafunc setP( v, m, a )
+ *    @luatparam number a Angle to set, in radians.
+ * @luafunc setP
  */
 static int vectorL_setP( lua_State *L )
 {
-   Vector2d *v1;
+   vec2 *v1;
    double m, a;
 
    /* Get parameters. */
    v1 = luaL_checkvector(L,1);
    m  = luaL_checknumber(L,2);
-   a  = luaL_checknumber(L,3)/180.*M_PI;
+   a  = luaL_checknumber(L,3);
 
-   vect_pset( v1, m, a );
+   vec2_pset( v1, m, a );
    return 0;
 }
 
@@ -554,29 +648,22 @@ static int vectorL_setP( lua_State *L )
  * @usage my_vec:dist( your_vec ) -- Gets distance from both vectors (your_vec - my_vec).
  *
  *    @luatparam Vec2 v Vector to act as origin.
- *    @luatparam Vec2 v2 Vector to get distance from, uses origin (0,0) if not set.
+ *    @luatparam[opt=vec2.new()] Vec2 v2 Vector to get distance from, uses origin (0,0) if not set.
  *    @luatreturn number The distance calculated.
- * @luafunc dist( v, v2 )
+ * @luafunc dist
  */
 static int vectorL_distance( lua_State *L )
 {
-   Vector2d *v1, *v2;
    double dist;
-
-   /* Get self. */
-   v1 = luaL_checkvector(L,1);
+   const vec2 *v1 = luaL_checkvector(L,1);
 
    /* Get rest of parameters. */
-   v2 = NULL;
-   if (lua_gettop(L) > 1) {
-      v2 = luaL_checkvector(L,2);
+   if (!lua_isnoneornil(L,2)) {
+      const vec2 *v2 = luaL_checkvector(L,2);
+      dist = vec2_dist(v1, v2);
    }
-
-   /* Get distance. */
-   if (v2 == NULL)
-      dist = vect_odist(v1);
    else
-      dist = vect_dist(v1, v2);
+      dist = vec2_odist(v1);
 
    /* Return the distance. */
    lua_pushnumber(L, dist);
@@ -590,29 +677,22 @@ static int vectorL_distance( lua_State *L )
  * @usage my_vec:dist2( your_vec ) -- Gets squared distance from both vectors (your_vec - my_vec)^2.
  *
  *    @luatparam Vec2 v Vector to act as origin.
- *    @luatparam Vec2 v2 Vector to get squared distance from, uses origin (0,0) if not set.
+ *    @luatparam[opt=vec2.new()] Vec2 v2 Vector to get squared distance from, uses origin (0,0) if not set.
  *    @luatreturn number The distance calculated.
- * @luafunc dist2( v, v2 )
+ * @luafunc dist2
  */
 static int vectorL_distance2( lua_State *L )
 {
-   Vector2d *v1, *v2;
    double dist2;
-
-   /* Get self. */
-   v1 = luaL_checkvector(L,1);
+   const vec2 *v1 = luaL_checkvector(L,1);
 
    /* Get rest of parameters. */
-   v2 = NULL;
-   if (lua_gettop(L) > 1) {
-      v2 = luaL_checkvector(L,2);
+   if (!lua_isnoneornil(L,2)) {
+      const vec2 *v2 = luaL_checkvector(L,2);
+      dist2 = vec2_dist2(v1, v2);
    }
-
-   /* Get distance. */
-   if (v2 == NULL)
-      dist2 = vect_odist2(v1);
    else
-      dist2 = vect_dist2(v1, v2);
+      dist2 = vec2_odist2(v1);
 
    /* Return the distance. */
    lua_pushnumber(L, dist2);
@@ -623,14 +703,96 @@ static int vectorL_distance2( lua_State *L )
  * @brief Gets the modulus of the vector.
  *    @luatparam Vec2 v Vector to get modulus of.
  *    @luatreturn number The modulus of the vector.
- * @luafunc mod(v)
+ * @luafunc mod
  */
 static int vectorL_mod( lua_State *L )
 {
-   Vector2d *v;
-
-   v = luaL_checkvector(L,1);
+   const vec2 *v = luaL_checkvector(L,1);
    lua_pushnumber(L, VMOD(*v));
    return 1;
 }
 
+/**
+ * @brief Gets the angle of the vector.
+ *    @luatparam Vec2 v Vector to get angle of.
+ *    @luatreturn number The angle of the vector.
+ * @luafunc angle
+ */
+static int vectorL_angle( lua_State *L )
+{
+   const vec2 *v = luaL_checkvector(L,1);
+   lua_pushnumber(L, VANGLE(*v));
+   return 1;
+}
+
+/**
+ * @brief Normalizes a vector.
+ *    @luatparam Vec2 v Vector to normalize.
+ *    @luatparam[opt=1] number n Length to normalize the vector to.
+ *    @luatreturn Vec2 Normalized vector.
+ * @luafunc normalize
+ */
+static int vectorL_normalize( lua_State *L )
+{
+   vec2 *v = luaL_checkvector(L,1);
+   double n = luaL_optnumber(L,2,1.);
+   double m = n/MAX(VMOD(*v),1e-6);
+   v->x *= m;
+   v->y *= m;
+   lua_pushvector(L, *v);
+   return 1;
+}
+
+/**
+ * @brief Sees if two line segments collide.
+ *
+ *    @luatparam Vec2 s1 Start point of the first segment.
+ *    @luatparam Vec2 e1 End point of the first segment.
+ *    @luatparam Vec2 s2 Start point of the second segment.
+ *    @luatparam Vec2 e2 End point of the second segment.
+ *    @luatreturn integer 0 if they don't collide, 1 if they collide on a point, 2 if they are parallel, and 3 if they are coincident.
+ * @luafunc collideLineLine
+ */
+static int vectorL_collideLineLine( lua_State *L )
+{
+   const vec2 *s1 = luaL_checkvector(L,1);
+   const vec2 *e1 = luaL_checkvector(L,2);
+   const vec2 *s2 = luaL_checkvector(L,3);
+   const vec2 *e2 = luaL_checkvector(L,4);
+   vec2 crash;
+   int ret = CollideLineLine( s1->x, s1->y, e1->x, e1->y, s2->x, s2->y, e2->x, e2->y, &crash );
+   lua_pushinteger( L, ret );
+   lua_pushvector( L, crash );
+   return 2;
+}
+
+/**
+ * @brief Computes the intersection of a line segment and a circle.
+ *
+ *    @luatparam Vector center Center of the circle.
+ *    @luatparam number radius Radius of the circle.
+ *    @luatparam Vector p1 First point of the line segment.
+ *    @luatparam Vector p2 Second point of the line segment.
+ *    @luatreturn Vector|nil First point of collision or nil if no collision.
+ *    @luatreturn Vector|nil Second point of collision or nil if single-point collision.
+ * @luafunc collideCircleLine
+ */
+static int vectorL_collideCircleLine( lua_State *L )
+{
+   const vec2 *center, *p1, *p2;
+   vec2 crash[2];
+   double radius;
+
+   center = luaL_checkvector( L, 1 );
+   radius = luaL_checknumber( L, 2 );
+   p1     = luaL_checkvector( L, 3 );
+   p2     = luaL_checkvector( L, 4 );
+
+   int cnt = CollideLineCircle( p1, p2, center, radius, crash );
+   if (cnt>0)
+      lua_pushvector( L, crash[0] );
+   if (cnt>1)
+      lua_pushvector( L, crash[1] );
+
+   return cnt;
+}
