@@ -1,19 +1,17 @@
 /*
  * See Licensing and Copyright notice in naev.h
  */
+#pragma once
 
-
-#ifndef NTIME_H
-#  define NTIME_H
-
-
+/** @cond */
 #include <stdint.h>
+/** @endcond */
 
+#define TIME_PRI        PRIi64
 
 #define NT_CYCLE_PERIODS   (5000)      /**< periods in a cycle */
 #define NT_PERIOD_SECONDS   (10000)     /**< seconds in a period */
 
-
 typedef int64_t ntime_t;         /**< Core time type. */
 
 /* Create. */
@@ -42,6 +40,3 @@ void ntime_incLagged( ntime_t t );
 /* misc */
 void ntime_refresh (void);
 void ntime_allowUpdate( int enable );
-
-
-#endif /* NTIME_H */
