@@ -1,16 +1,8 @@
 /*
  * See Licensing and Copyright notice in naev.h
  */
-
-
-
-#ifndef OPTIONS_H
-#  define OPTIONS_H
-
+#pragma once
 
 void opt_menu (void);
 void opt_resize (void);
 int opt_setVideoMode( int w, int h, int fullscreen, int confirm );
-
-
-#endif /* OPTIONS_H */
