@@ -1,12 +1,9 @@
 /*
  * See Licensing and Copyright notice in naev.h
  */
+#pragma once
 
-
-
-#ifndef SLOTPROPERTY_H
-#  define SLOTPROPERTY_H
-
+#include "opengl_tex.h"
 
 /* Load/exit. */
 int sp_load (void);
@@ -18,7 +15,5 @@ const char *sp_display( unsigned int sp );
 const char *sp_description( unsigned int sp );
 int sp_required( unsigned int spid );
 int sp_exclusive( unsigned int spid );
-
-
-#endif /* SLOTPROPERTY_H */
-
+int sp_locked( unsigned int spid );
+const glTexture * sp_icon( unsigned int spid );
