@@ -1,16 +1,10 @@
 /*
  * See Licensing and Copyright notice in naev.h
  */
+#pragma once
 
-
-
-#ifndef WGT_IMAGE_H
-#  define WGT_IMAGE_H
-
-
-#include "font.h"
 #include "colour.h"
-
+#include "font.h"
 
 /**
  * @brief The image widget data
@@ -24,22 +18,17 @@ typedef struct WidgetImageData_{
    int nlayers; /**< Total number of layers. */
 } WidgetImageData;
 
-
 /* Required functions. */
-void window_addImage( const unsigned int wid,
+void window_addImage( unsigned int wid,
       const int x, const int y, /* position */
       const int w, const int h, /* dimensions */
-      char* name, glTexture* image, int border ); /* label and image itself */
+      char* name, const glTexture* image, int border ); /* label and image itself */
 
 /* Misc functions. */
-void window_modifyImage( const unsigned int wid,
-      char* name, glTexture* image, int w, int h );
-void window_imgColour( const unsigned int wid,
+void window_modifyImage( unsigned int wid,
+      char* name, const glTexture* image, int w, int h );
+void window_imgColour( unsigned int wid,
       char* name, const glColour* colour );
-glTexture* window_getImage( const unsigned int wid, char* name );
-void window_modifyImageLayers( const unsigned int wid,
+glTexture* window_getImage( unsigned int wid, char* name );
+void window_modifyImageLayers( unsigned int wid,
       char* name, glTexture** layers, int n );
-
-
-#endif /* WGT_IMAGE_H */
-
