@@ -1,18 +1,21 @@
 #!/bin/bash
 
-# STEAM DEPLOYMENT SCRIPT FOR NAEV
-# Requires SteamCMD to be installed within a Github Actions ubuntu-latest runner.
-#
-# Written by Jack Greiner (ProjectSynchro on Github: https://github.com/ProjectSynchro/) 
-#
-# This script should be run after downloading all build artefacts
-# If -n is passed to the script, a nightly build will be generated
-# and uploaded to Steam
-#
-# Pass in [-d] [-n] (set this for nightly builds) [-p] (set this for pre-release builds.) [-c] (set this for CI testing) -s <SCRIPTROOT> (Sets path to look for additional steam scripts.) -t <TEMPPATH> (Steam build artefact location) -o <STEAMPATH> (Steam dist output directory)
-
 set -e
 
+usage() {
+    cat <<EOF
+STEAM DEPLOYMENT SCRIPT FOR NAEV
+Requires SteamCMD to be installed within a Github Actions ubuntu-latest runner.
+
+This script should be run after downloading all build artefacts
+If -n is passed to the script, a nightly build will be generated
+and uploaded to Steam
+
+Pass in [-d] [-n] (set this for nightly builds) [-p] (set this for pre-release builds.) [-c] (set this for CI testing) -s <SCRIPTROOT> (Sets path to look for additional steam scripts.) -t <TEMPPATH> (Steam build artefact location) -o <STEAMPATH> (Steam dist output directory)
+EOF
+    exit 1
+}
+
 # Defaults
 NIGHTLY="false"
 PRERELEASE="false"
@@ -42,9 +45,28 @@ while getopts dnpc:s:t:o: OPTION "$@"; do
     o)
         STEAMPATH="${OPTARG}"
         ;;
+    *)
+        usage
+        ;;
     esac
 done
 
+retry() {
+    local -r -i max_attempts="$1"; shift
+    local -i attempt_num=1
+    until "$@"
+    do
+        if ((attempt_num==max_attempts))
+        then
+            echo "Attempt $attempt_num failed and there are no more attempts left!"
+            return 1
+        else
+            echo "Attempt $attempt_num failed! Trying again in $attempt_num seconds..."
+            sleep $((attempt_num++))
+        fi
+    done
+}
+
 # Make Steam dist path if it does not exist
 mkdir -p "$STEAMPATH"/content/lin64
 mkdir -p "$STEAMPATH"/content/macos
@@ -58,23 +80,22 @@ cp -v -r "$SCRIPTROOT"/scripts "$STEAMPATH"
 # Move Linux binary and set as executable
 cp -v "$TEMPPATH"/naev-steamruntime/naev.x64 "$STEAMPATH"/content/lin64
 chmod +x "$STEAMPATH"/content/lin64/naev.x64
-          
+
 # Move macOS bundle to deployment location
 unzip "$TEMPPATH/naev-macos/naev-macos.zip" -d "$STEAMPATH/content/macos/"
 
-# Unzip Windows binary and DLLs and move to deployment location
-tar -Jxf "$TEMPPATH/naev-win64/steam-win64.tar.xz" -C "$STEAMPATH/content/win64"
-mv "$STEAMPATH"/content/win64/naev*.exe "$STEAMPATH/content/win64/naev.exe"
+# Untar Windows binary and DLLs
+tar -Jxf "$TEMPPATH/naev-win64/naev-windows.tar.xz" -C "$STEAMPATH/content/win64"
 
 # Move data to deployment location
-tar -Jxf "$TEMPPATH/naev-ndata/steam-ndata.tar.xz" --strip=1 -C "$STEAMPATH/content/ndata"
+tar -Jxf "$TEMPPATH/naev-ndata/naev-ndata.tar.xz" -C "$STEAMPATH/content/ndata"
 
 # Runs STEAMCMD, and builds the app as well as all needed depots.
 
 if [ "$DRYRUN" == "false" ]; then
 
     # Trigger 2FA request and get 2FA code
-    steamcmd +login $STEAMCMD_USER $STEAMCMD_PASS +quit || true
+    steamcmd +login "$STEAMCMD_USER" "$STEAMCMD_PASS" +quit || true
 
     # Wait a bit for the email to arrive
     sleep 60s
@@ -83,21 +104,20 @@ if [ "$DRYRUN" == "false" ]; then
 
     if [ "$NIGHTLY" == "true" ]; then
         # Run steam upload with 2fa key
-        steamcmd +login $STEAMCMD_USER $STEAMCMD_PASS $STEAMCMD_TFA +run_app_build_http "$STEAMPATH/scripts/app_build_598530_nightly.vdf" +quit
+        retry 5 steamcmd +login "$STEAMCMD_USER" "$STEAMCMD_PASS" "$STEAMCMD_TFA" +run_app_build_http "$STEAMPATH/scripts/app_build_598530_nightly.vdf" +quit
     else
-        if [ "$PRERELEASE" == "true" ]; then 
+        if [ "$PRERELEASE" == "true" ]; then
             # Run steam upload with 2fa key
-            steamcmd +login $STEAMCMD_USER $STEAMCMD_PASS $STEAMCMD_TFA +run_app_build_http "$STEAMPATH/scripts/app_build_598530_prerelease.vdf" +quit
+            retry 5 steamcmd +login "$STEAMCMD_USER" "$STEAMCMD_PASS" "$STEAMCMD_TFA" +run_app_build_http "$STEAMPATH/scripts/app_build_598530_prerelease.vdf" +quit
 
-        elif [ "$PRERELEASE" == "false" ]; then 
+        elif [ "$PRERELEASE" == "false" ]; then
             mkdir -p "$STEAMPATH"/content/soundtrack
             # Move soundtrack stuff to deployment area
-            cp "$TEMPPATH"/naev-steam-soundtrack/*.mp3 "$STEAMPATH/content/soundtrack"
-            cp "$TEMPPATH"/naev-steam-soundtrack/*.png "$STEAMPATH/content/soundtrack"
+            cp "$TEMPPATH"/naev-steam-soundtrack/*.* "$STEAMPATH/content/soundtrack"
 
             # Run steam upload with 2fa key
-            steamcmd +login $STEAMCMD_USER $STEAMCMD_PASS $STEAMCMD_TFA +run_app_build_http "$STEAMPATH/scripts/app_build_598530_release.vdf" +quit
-            steamcmd +login $STEAMCMD_USER $STEAMCMD_PASS +run_app_build_http "$STEAMPATH/scripts/app_build_1411430_soundtrack.vdf" +quit
+            retry 5 steamcmd +login "$STEAMCMD_USER" "$STEAMCMD_PASS" "$STEAMCMD_TFA" +run_app_build_http "$STEAMPATH/scripts/app_build_598530_release.vdf" +quit
+            retry 5 steamcmd +login "$STEAMCMD_USER" "$STEAMCMD_PASS" +run_app_build_http "$STEAMPATH/scripts/app_build_1411430_soundtrack.vdf" +quit
 
         else
             echo "Something went wrong determining if this is a PRERELEASE or not."
@@ -115,14 +135,13 @@ elif [ "$DRYRUN" == "true" ]; then
         echo "steamcmd nightly build"
         ls -l -R "$STEAMPATH"
     else
-        if [ "$PRERELEASE" == "true" ]; then 
+        if [ "$PRERELEASE" == "true" ]; then
             # Run steam upload with 2fa key
             echo "steamcmd PRERELEASE build"
             ls -l -R "$STEAMPATH"
-        elif [ "$PRERELEASE" == "false" ]; then 
+        elif [ "$PRERELEASE" == "false" ]; then
             # Move soundtrack stuff to deployment area
-            cp -v "$TEMPPATH"/naev-steam-soundtrack/*.mp3 "$STEAMPATH/content/soundtrack"
-            cp -v "$TEMPPATH"/naev-steam-soundtrack/*.png "$STEAMPATH/content/soundtrack"
+            echo "copy soundtrack files"
 
             # Run steam upload with 2fa key
             echo "steamcmd release build"
